from __future__ import division
import math

import numpy as np
from numpy import pi

import numdifftools.fornberg as ndf

@np.vectorize
def CxDerivative(f, z0, n=1, contour=None, absIntegrationTol=1e-10, verbose=False):
	r"""
	Compute the derivaive of an analytic function using Cauchy's
	Integral Formula for Derivatives.

	.. math::

		f^{(n)}(z_0) = \frac{n!}{2\pi i} \oint_C \frac{f(z)}{(z-z_0)^{n+1}} dz

	Parameters
	----------
	f : function
		Function of a single variable f(x).
	z0 : complex
		Point to evaluate the derivative at.
	n : int
		The order of the derivative to evaluate.
	contour : :class:`Contour <cxroots.Contour.Contour>`, optional
		The contour, C, in the complex plane which encloses the point z0.
		By default the contour is the circle |z-z_0|=1e-3.
	absIntegrationTol : float, optional
		The absolute tolerance required of the integration routine.
	verbose : bool, optional
		If True runtime information will be printed.  False be default.

	Returns
	-------
	f^{(n)}(z0) : complex
		The nth derivative of f evaluated at z0
	"""
	if contour is None:
		from .contours.Circle import Circle
		C = lambda z0: Circle(z0, 1e-3)
	else:
		C = lambda z0: contour

	integrand = lambda z: f(z)/(z-z0)**(n+1)
	integral = C(z0).integrate(integrand, absTol=absIntegrationTol, verbose=verbose)
	return integral * math.factorial(n)/(2j*pi)


def find_multiplicity(root, f, df=None, rootErrTol=1e-10, verbose=False):
	"""
	Find the multiplicity of a given root of f by computing the
	derivatives of f, f^{(1)}, f^{(2)}, ... until
	|f^{(n)}(root)|>rootErrTol.  The multiplicity of the root is then
	equal to n.  The derivative is calculated with `numdifftools <http://numdifftools.readthedocs.io/en/latest/api/numdifftools.html#numdifftools.fornberg.derivative>`_
	which employs a method due to Fornberg.

	Parameters
	----------
	root : complex
		A root of f, f(root)=0.
	f : function
		An analytic function of a single complex variable such that
		f(root)=0.
	df : function, optional
		The first derivative of f.  If not known then df=None.
	contour : Contour, optional
		The integration contour used to evaluate the derivatives.
	rootErrTol : float, optional
		It will be assumed that f(z)=0 if numerically |f(z)|<rootErrTol.
	verbose : bool, optional
		If True runtime information will be printed.  False be default.

	Returns
	-------
	multiplicity : int
		The multiplicity of the given root.
	"""
	if abs(f(root)) > rootErrTol:
		raise ValueError("""
			The provided 'root' is not a root of the given function f.
			Specifically, %f = abs(f(root)) > rootErrTol = %f
			"""%(abs(f(root)), rootErrTol))

	n = 1
	while True:
		if df is not None:
			if n==1:
				err = abs(df(root))
			else:
				# ndf.derivative returns an array [f, f', f'', ...]
				err = abs(ndf.derivative(df, root, n-1)[n-1])
		else:
			err = abs(ndf.derivative(f, root, n)[n])

		if verbose:
			print('n', n, '|df^(n)|', err)

		if err > rootErrTol:
			break

		n += 1

	return n




from __future__ import division
import numpy as np
from scipy import pi, exp

from ..Contour import Contour
from ..Paths import ComplexLine, ComplexArc

class AnnulusSector(Contour):
	"""
	A sector of an annulus in the complex plane.

	Parameters
	----------
	center : complex
		The center of the annulus sector.
	radii : tuple
		Tuple of length two of the form (inner_radius, outer_radius)
	phiRange : tuple
		Tuple of length two of the form (phi0, phi1).
		The segment of the contour containing inner and outer circular
		arcs will be joined, counter clockwise from phi0 to phi1.

	Examples
	--------
	.. plot::
		:include-source:

		from numpy import pi
		from cxroots import AnnulusSector
		annulusSector = AnnulusSector(center=0.2, radii=(0.5, 1.25), phiRange=(-pi/4, pi/4))
		annulusSector.show()

	.. plot::
		:include-source:

		from numpy import pi
		from cxroots import AnnulusSector
		annulusSector = AnnulusSector(center=0.2, radii=(0.5, 1.25), phiRange=(pi/4, -pi/4))
		annulusSector.show()
	"""
	def __init__(self, center, radii, phiRange):
		self.center = center
		self.axisName = ('r', 'phi')

		if phiRange[0] > phiRange[1]:
			phiRange = (phiRange[0], phiRange[1]+2*pi)

		phi0, phi1 = self.phiRange = phiRange

		# r > 0
		r0, r1 = self.radii = radii
		if r0 < 0 or r1 <= 0:
			raise ValueError('Radius > 0')

		# verticies [[radius0,phi0],[radius0,phi1],[radius1,phi1],[radius0,phi1]]
		self.z1 = z1 = center + r0*exp(1j*phi0)
		self.z2 = z2 = center + r1*exp(1j*phi0)
		self.z3 = z3 = center + r1*exp(1j*phi1)
		self.z4 = z4 = center + r0*exp(1j*phi1)

		segments = [ComplexLine(z1,z2),
					ComplexArc(center,r1,phi0,phi1-phi0),
					ComplexLine(z3,z4),
					ComplexArc(center,r0,phi1,phi0-phi1)]

		super(AnnulusSector, self).__init__(segments)

	def __str__(self):
		return 'Annulus sector: center={center.real:.3f}{center.imag:+.3f}i, r0={radii[0]:.3f}, r1={radii[1]:.3f}, phi0={phiRange[0]:.3f}, phi1={phiRange[1]:.3f}'.format(center=self.center, radii=self.radii, phiRange=self.phiRange)

	@property
	def centralPoint(self):
		# get the central point within the contour
		r = (self.radii[0] + self.radii[1])/2
		phi = (self.phiRange[0] + self.phiRange[1])/2
		return r*exp(1j*phi)

	@property
	def area(self):
		return (self.radii[1]**2 - self.radii[0]**2)*abs(self.phiRange[1] - self.phiRange[0])%(2*pi)/2

	def contains(self, z):
		""" Returns True if the point z lies within the contour, False if otherwise """
		angle = np.angle(z - self.center)%(2*pi) # np.angle maps to [-pi,pi]
		radiusCorrect = self.radii[0] < abs(z - self.center) < self.radii[1]

		phi = np.mod(self.phiRange, 2*pi)
		if phi[0] > phi[1]:
			angleCorrect = phi[0] < angle <= 2*pi or 0 <= angle < phi[1]
		else:
			angleCorrect = phi[0] < angle < phi[1]

		return radiusCorrect and angleCorrect

	def subdivide(self, axis, divisionFactor=0.5):
		"""
		Subdivide the contour

		Parameters
		----------
		axis : str, can be either 'r' or 'phi'
			The axis along which the line subdividing the contour is a constant.
		divisionFactor : float in range (0,1), optional
			Determines the point along 'axis' at which the line dividing the box is placed

		Returns
		-------
		box1 : AnnulusSector
			If axis is 'r' then phiRange and the inner radius is the same as original AnnulusSector
			with the outer radius determined by the divisionFactor.
			If axis is 'phi' then the radii and phiRange[0] is the same as the original AnnulusSector
			with phiRange[1] determined by the divisionFactor.
		box2 : AnnulusSector
			If axis is 'r' then phiRange and the outer radius is the same as original AnnulusSector
			with the inner radius determined equal to the outer radius of box1.
			If axis is 'phi' then the radii and phiRange[1] is the same as the original AnnulusSector
			with phiRange[0] equal to phiRange[1] of box1.
		"""
		r0, r1 = self.radii
		phi0, phi1 = self.phiRange
		if axis == 0 or axis == self.axisName[0]:
			divisionPoint = r0 + divisionFactor*(r1-r0)
			box1 = AnnulusSector(self.center, [r0, divisionPoint], self.phiRange)
			box2 = AnnulusSector(self.center, [divisionPoint, r1], self.phiRange)

			# reuse line segments from original box where possible
			# this allows the cached integrals to be used
			box1.segments[3] = self.segments[3]
			box2.segments[1] = self.segments[1]
			box1.segments[1]._reversePath = box2.segments[3]
			box2.segments[3]._reversePath = box1.segments[1]

		elif axis == 1 or axis == self.axisName[1]:
			divisionPoint = phi0 + divisionFactor*(phi1-phi0)
			box1 = AnnulusSector(self.center, self.radii, [phi0, divisionPoint])
			box2 = AnnulusSector(self.center, self.radii, [divisionPoint, phi1])

			box1.segments[0] = self.segments[0]
			box2.segments[2] = self.segments[2]
			box1.segments[2]._reversePath = box2.segments[0]
			box2.segments[0]._reversePath = box1.segments[2]

		for box in [box1, box2]:
			box._createdBySubdivisionAxis = axis
			box._parentBox = self
		self._childBoxes = [box1, box2]

		return box1, box2

	def randomPoint(self):
		"""Returns a random point inside the contour of the AnnulusSector."""
		r = np.random.uniform(*self.radii)
		phiRange = np.mod(self.phiRange, 2*pi)
		if phiRange[0] > phiRange[1]:
			phi = random.choice([np.random.uniform(phiRange[0], 2*pi),
								 np.random.uniform(0, phiRange[1])])
		else:
			phi = np.random.uniform(*phiRange)

		return r*exp(1j*phi) + self.center



from .contours.Circle import Circle
from .contours.Annulus import Annulus
from .contours.AnnulusSector import AnnulusSector
from .contours.Rectangle import Rectangle

from .RootFinder import find_roots
from .DemoRootFinder import demo_find_roots
from .CountRoots import count_roots
from .Derivative import CxDerivative
from .Derivative import find_multiplicity




import pytest

import numpy as np
from scipy import pi
from numpy.random import uniform

from cxroots.Paths import ComplexArc, ComplexLine

# have a common seed for each testing process
from datetime import date
today = date.today()
np.random.seed(today.year*today.month*today.day)

@pytest.mark.parametrize('a', uniform(-10, 10, size=3) + 1j*uniform(-10, 10, size=3))
@pytest.mark.parametrize('b', uniform(-10, 10, size=3) + 1j*uniform(-10, 10, size=3))
@pytest.mark.parametrize('P', uniform(-10, 10, size=3) + 1j*uniform(-10, 10, size=3))
def test_distance_line(a, b, P):
	t = np.linspace(0,1,100001)
	C = ComplexLine(a, b)
	assert C.distance(P) == pytest.approx(np.min(np.abs(C(t) - P)), abs=1e-6)


@pytest.mark.parametrize('z0', uniform(-10, 10, size=3) + 1j*uniform(-10, 10, size=3))
@pytest.mark.parametrize('R', uniform(0, 10, size=3))
@pytest.mark.parametrize('t0', uniform(0, 2*pi, size=3))
@pytest.mark.parametrize('dt', uniform(-2*pi, 2*pi, size=3))
@pytest.mark.parametrize('P', uniform(-10, 10, size=3) + 1j*uniform(-10, 10, size=3))
def test_distance_arc(z0, R, t0, dt, P):
	z0 = 0
	t = np.linspace(0,1,100001)
	C = ComplexArc(z0, R, t0, dt)
	assert C.distance(P) == pytest.approx(np.min(np.abs(C(t) - P)), abs=1e-6)



"""
References
----------
[DSZ] "Locating all the Zeros of an Analytic Function in one Complex Variable"
	M.Dellnitz, O.Schutze, Q.Zheng, J. Compu. and App. Math. (2002), Vol.138, Issue 2
[DL] "A Numerical Method for Locating the Zeros of an Analytic function", 
	L.M.Delves, J.N.Lyness, Mathematics of Computation (1967), Vol.21, Issue 100
[KB] "Computing the zeros of analytic functions" by Peter Kravanja, Marc Van Barel, Springer 2000
"""

import unittest
import pytest
import numpy as np
from scipy import pi, sqrt, exp, sin, cos

from cxroots import Circle, Rectangle, Annulus
from cxroots.tests.ApproxEqual import roots_approx_equal
from cxroots.Derivative import find_multiplicity

class RootfindingTests(object):
	def test_rootfinding_romb_df(self):
		roots_approx_equal(self.C.roots(self.f, self.df, intMethod='romb', verbose=True), (self.roots, self.multiplicities), decimal=10)

	def test_rootfinding_romb_f(self):
		roots_approx_equal(self.C.roots(self.f, intMethod='romb', verbose=True), (self.roots, self.multiplicities), decimal=10)

	def test_rootfinding_quad_df(self):
		roots_approx_equal(self.C.roots(self.f, self.df, intMethod='quad', verbose=True), (self.roots, self.multiplicities), decimal=10)

	def test_rootfinding_quad_f(self):
		roots_approx_equal(self.C.roots(self.f, intMethod='quad', verbose=True), (self.roots, self.multiplicities), decimal=10)


class MultiplicityTests(object):
	def test_multiplicity_f(self):
		# Check that if only the root is given then the multiplcity could be computed
		for i, root in enumerate(self.roots):
			assert find_multiplicity(root, self.f, df=None, verbose=True) == self.multiplicities[i]

	def test_multiplicity_df(self):
		# Check that if only the root is given then the multiplcity could be computed
		for i, root in enumerate(self.roots):
			assert find_multiplicity(root, self.f, df=self.df, verbose=True) == self.multiplicities[i]


class TestRootfinding_noRoots(unittest.TestCase, RootfindingTests):
	def setUp(self):
		self.C = Annulus(1, [1,2])
		self.f = lambda z: (z-1)**3
		self.df = lambda z: 3*(z-1)**2

		self.roots = []
		self.multiplicities = []

class TestRootfinding_poly1(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		self.C = Rectangle([-2,2],[-2,2])
		self.f = lambda z: z**3 * (z-1.2)**2
		self.df = lambda z: 3*(z)**2 * (z-1.2)**2 + 2*z**3 * (z-1.2)

		self.roots = [0, 1.2]
		self.multiplicities = [3,2]

class TestRootfinding_poly2(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		self.C = Annulus(0, [0.5, 2.5])
		self.f = lambda z: (z-2)**2*(z-1)**5
		self.df = lambda z: 2*(z-2)*(z-1)**5 + 5*(z-2)**2*(z-1)**4

		self.roots = [1, 2]
		self.multiplicities = [5,2]

class TestRootfinding_141(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.4.1 from [KB]
		self.C = Circle(0,3)
		e = 1e-2
		self.f  = lambda z: (z-e)*(1+(z-sqrt(3))**2)
		self.df = lambda z: (1+(z-sqrt(3))**2) + (z-e)*2*(z-sqrt(3))

		self.roots = [e, sqrt(3)+1j, sqrt(3)-1j]
		self.multiplicities = [1,1,1]

class TestRootfinding_142(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.4.2 from [KB]
		self.C = Circle(0,2)
		self.f  = lambda z: exp(3*z) + 2*z*cos(z) - 1
		self.df = lambda z: 3*exp(3*z) + 2*cos(z) - 2*z*sin(z)

		self.roots = [0,
				 	  -1.844233953262213, 
				 	  0.5308949302929305 + 1.33179187675112098j,
				 	  0.5308949302929305 - 1.33179187675112098j]
		self.multiplicities = [1,1,1,1]

class TestRootfinding_142b(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.4.2 from [KB] with a rectangular initial contour
		self.C = Rectangle([-2,2],[-2,2])
		self.f  = lambda z: exp(3*z) + 2*z*cos(z) - 1
		self.df = lambda z: 3*exp(3*z) + 2*cos(z) - 2*z*sin(z)

		self.roots = [0,
					  -1.844233953262213, 
					  0.5308949302929305 + 1.33179187675112098j,
					  0.5308949302929305 - 1.33179187675112098j]
		self.multiplicities = [1,1,1,1]

class TestRootfinding_143(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.4.3 from [KB]
		self.C = Circle(0,5)
		self.f  = lambda z: z**2*(z-1)*(z-2)*(z-3)*(z-4)+z*sin(z)
		self.df = lambda z: 2*z*(3*z**4-25*z**3+70*z**2-75*z+24)+sin(z)+z*cos(z)

		self.roots = [0,
				 	  1.18906588973011365517521756, 
				 	  1.72843498616506284043592924,
				 	  3.01990732809571222812005354,
				 	  4.03038191606046844562845941]
		self.multiplicities = [2,1,1,1,1]

class TestRootfinding_144(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.4.4 from [KB]
		self.C = Circle(0,3)
		self.f  = lambda z: (z*(z-2))**2*(exp(2*z)*cos(z)+z**3-1-sin(z))
		self.df = lambda z: 2*z*(z-2)**2*(exp(2*z)*cos(z)+z**3-1-sin(z))+2*(z-2)*z**2*(exp(2*z)*cos(z)+z**3-1-sin(z))+(z*(z-2))**2*(2*exp(2*z)*cos(z)-exp(2*z)*sin(z)+3*z**2-cos(z))

		self.roots = [-0.4607141197289707542294459477 - 0.6254277693477682516688207854j,
				 -0.4607141197289707542294459477 + 0.6254277693477682516688207854j,
				 0,
				 2,
				 1.66468286974551654134568653]
		self.multiplicities = [1,1,3,2,1]

class TestRootfinding_145(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.4.5 from [KB]
		self.C = Circle(0,11)
		self.f  = lambda z: np.prod([z-k for k in range(1,11)], axis=0)
		self.df = lambda z: np.sum([np.prod([z-k for k in range(1,11) if k!=m], axis=0) for m in range(1,11)], axis=0)

		self.roots = [1,2,3,4,5,6,7,8,9,10]
		self.multiplicities = np.ones(10)

class TestRootfinding_145b(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.4.5 from [KB] with a rectangular initial contour
		self.C = Rectangle([-1,11],[-1,1])
		self.f  = lambda z: np.prod([z-k for k in range(1,11)], axis=0)
		self.df = lambda z: np.sum([np.prod([z-k for k in range(1,11) if k!=m], axis=0) for m in range(1,11)], axis=0)

		self.roots = [1,2,3,4,5,6,7,8,9,10]
		self.multiplicities = np.ones(10)

class TestRootfinding_151(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.5.1 from [KB]
		self.C = Rectangle([-2,2], [-2,3])
		self.f  = lambda z: exp(3*z) + 2*z*cos(z) - 1
		self.df = lambda z: 3*exp(3*z) + 2*cos(z) - 2*z*sin(z)

		self.roots = np.array([-1.84423395326221337491592440,
				 			    0,
				 			    0.5308949302929305274642203840 - 1.331791876751120981651544228j,
				 			    0.5308949302929305274642203840 + 1.331791876751120981651544228j], dtype=complex)
		self.multiplicities = [1,1,1,1]

	def test_rootfinding_b_df(self):
		roots_approx_equal(self.C.roots(self.f, self.df, verbose=True, M=2), (self.roots, self.multiplicities), decimal=12)

	def test_rootfinding_b_f(self):
		roots_approx_equal(self.C.roots(self.f, verbose=True, M=2), (self.roots, self.multiplicities), decimal=12)

class TestRootfinding_152(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.5.2 from [KB]
		self.C = Rectangle([-0.5,5.5], [-0.5,1.5])
		self.f  = lambda z: z**2*(z-1)*(z-2)*(z-3)*(z-4)+z*sin(z)
		self.df = lambda z: 2*z*(3*z**4-25*z**3+70*z**2-75*z+24)+sin(z)+z*cos(z)

		self.roots = [0,
				 	  1.18906588973011365517521756,
				 	  1.72843498616506284043592924,
				 	  3.01990732809571222812005354,
				 	  4.03038191606046844562845941]
		self.multiplicities = [2,1,1,1,1]

class TestRootfinding_153(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		# Ex 1.5.3 from [KB]
		self.C = Rectangle([-1,3], [-1,1])
		self.f  = lambda z: (z*(z-2))**2 * (exp(2*z)*cos(z)+z**3-1-sin(z))
		self.df = lambda z: 2*z*(z-2)**2 * (exp(2*z)*cos(z)+z**3-1-sin(z)) + 2*z**2*(z-2) * (exp(2*z)*cos(z)+z**3-1-sin(z)) + (z*(z-2))**2 * (2*exp(2*z)*cos(z)-exp(2*z)*sin(z)+3*z**2-cos(z))

		self.roots = [0, 2,
					  1.66468286974551654134568653,
					  -0.4607141197289707542294459477 - 0.6254277693477682516688207854j,
					  -0.4607141197289707542294459477 + 0.6254277693477682516688207854j]
		self.multiplicities = [3,2,1,1,1]

def test_reevaluation_of_N():
	from cxroots import Circle
	C = Circle(0,2)
	f = lambda z: (z-1)*(z-0.2)**2

	roots 		   = [1, 0.2]
	multiplicities = [1, 2]
	roots_approx_equal(C.roots(f, NIntAbsTol=10, intMethod='romb', verbose=True), (roots, multiplicities))

class TestIntroduction(unittest.TestCase, RootfindingTests, MultiplicityTests):
	def setUp(self):
		self.C = Circle(0,3)
		self.f = lambda z: (z*(z+2))**2 * (exp(2*z)*cos(z)-1-sin(z)+z**5)
		self.df = lambda z: 2*(exp(2*z)*cos(z)-1-sin(z)+z**5)*(z**2*(z+2)+(z+2)**2*z) + (z*(z+2))**2*(2*exp(2*z)*cos(z)-exp(2*z)*sin(z)-cos(z)+5*z**4)

		self.roots = [0, -2,
					  2.23755778246706002284084684,
					  -0.6511140702635986824274097994 - 0.3904257190882864369857773146j,
					  -0.6511140702635986824274097994 + 0.3904257190882864369857773146j,
					  0.64857808095387581293067569277 - 1.35662268398824203963215495605j,
					  0.64857808095387581293067569277 + 1.35662268398824203963215495605j]
		self.multiplicities = [3,2,1,1,1,1,1]

def test_annular_combustion():
	from numpy import exp
	from cxroots import Rectangle

	A = -0.19435
	B = 1000.41
	C = 522463
	T = 0.005

	f = lambda z: z**2 + A*z + B*exp(-T*z) + C
	df = lambda z: 2*z + A - B*T*exp(-T*z)

	rectangle = Rectangle([-15000,5000], [-15000,15000])

	import warnings
	warnings.filterwarnings('error')
	roots = rectangle.roots(f, df, verbose=True, rootErrTol=1e-6)
	assert len(roots.roots) == 24


if __name__ == '__main__':
	unittest.main(verbosity=3)



from __future__ import division
import functools

import numpy as np
import scipy.linalg

from .CountRoots import prod

def approximate_roots(C, N, f, df=None, absTol=1e-12, relTol=1e-12,
	errStop=1e-10, divMin=3, divMax=15, m=2, rootTol=1e-8,
	intMethod='quad', callback=None, verbose=False):
	"""
	Approximate the roots and multiplcities of the function f within the
	contour C using the method of [KB]_.  The multiplicites are computed
	using eq. (21) in [SLV]_.

	Parameters
	----------
	C : :class:`~<cxroots.Contour.Contour>`
		The contour which encloses the roots of f the user wishes to find.
	N : int
		The number of roots (counting multiplicties) of f within C.
		This is the result of calling :meth:`~cxroots.Contour.Contour.count_roots`.
	f : function
		The function for which the roots are sought.  Must be a function
		of a single complex variable, z, which is analytic within C and
		has no poles or roots on the C.
	df : function, optional
		A function of a single complex variable which is the derivative
		of the function f(z). If df is not given then it will be
		approximated with a finite difference formula.
	absTol : float, optional
		Absolute error tolerance for integration.
	relTol : float, optional
		Relative error tolerance for integration.
	errStop : float, optional
		The number of distinct roots within a contour, n, is determined
		by checking if all the elements of a list of contour integrals
		involving formal orthogonal polynomials are sufficently close to
		zero, ie. that the absolute value of each element is < errStop.
		If errStop is too large/small then n may be smaller/larger than
		it actually is.
	divMin : int, optional
		If the Romberg integration method is used then divMin is the
		minimum number of divisions before the Romberg integration
		routine is allowed to exit.
	divMax : int, optional
		If the Romberg integration method is used then divMax is the
		maximum number of divisions before the Romberg integration
		routine exits.
	m : int, optional
		Only used if df=None and method='quad'.  The argument order=m is
		passed to numdifftools.Derivative and is the order of the error
		term in the Taylor approximation.  m must be even.
	rootTol : float, optional
		If any roots are within rootTol of one another then they will be
		treated as duplicates and removed.  This helps to alleviate the
		problem of errStop being too small.
	intMethod : {'quad', 'romb'}, optional
		If 'quad' then :func:`scipy.integrate.quad` is used to perform
		integration.  If 'romb' then Romberg integraion is performed
		instead.
	callback : function, optional
		Only used if intMethod is 'romb'.  Passed to :func:`~<cxroots.CountRoots.prod>`.
	verbose : bool, optional
		If True certain information regarding the rootfinding process
		will be printed.

	Returns
	-------
	tuple of complex
		The distinct roots of f within the contour C.
	tuple of float
		The corresponding multiplicites of the roots within C.  Should
		be integers but will not be automatically rounded here.

	References
	----------
	.. [KB] P. Kravanja and M. Van Barel. "Computing the Zeros of
		Anayltic Functions". Springer (2000)
	.. [SLV] E. Strakova, D. Lukas, P. Vodstrcil. "Finding Zeros of
		Analytic Functions and Local Eigenvalue Analysis Using Contour
		Integral Method in Examples". Mathematical Analysis and Numerical
		Mathematics, Vol. 15, 2, (2017)
	"""
	if verbose:
		print('Approximating roots in: ' + str(C))
		print('The number of roots, counting multiplcities, within this contour is', N)

	if N == 0:
		return (), ()

	product = functools.partial(prod, C, f, df,
		absTol=absTol, relTol=relTol, divMin=divMin, divMax=divMax,
		m=m, intMethod=intMethod, verbose=verbose, callback=callback)

	s = [N, product(lambda z: z)[0]]	# ordinary moments
	mu = s[1]/N
	phiZeros = [[],[mu]]

	def phi(i):
		if len(phiZeros[i]) == 0:
			return lambda z: np.ones_like(z)
		else:
			coeff = np.poly(phiZeros[i])
			return lambda z: np.polyval(coeff, z)

	# initialize G_{pq} = <phi_p, phi_q>
	G = np.zeros((N,N), dtype=np.complex128)
	G[0,0] = N # = <phi_0, phi_0> = <1,1>

	# initialize G1_{pq} = <phi_p, phi_1 phi_q>
	G1 = np.zeros((N,N), dtype=np.complex128)
	G1[0,0] = 0 # = <phi_0, phi_1 phi_0> = <1, z-mu> = s1-mu*N = 0

	r, t = 1, 0
	while r+t<N:
		### define FOP of degree r+t+1
		p = r+t
		G[p, 0:p+1] = [product(phi(p), phi(q))[0] for q in range(r+t+1)]
		G[0:p+1, p] = G[p, 0:p+1] # G is symmetric
		if verbose: print('G ', G[:p+1,:p+1])

		G1[p, 0:p+1] = [product(phi(p), lambda z: phi(1)(z)*phi(q)(z))[0] for q in range(r+t+1)]
		G1[0:p+1, p] = G1[p, 0:p+1] # G1 is symmetric
		if verbose: print('G1', G1[:p+1,:p+1])

		"""
		If any of the zeros of the FOP are outside of the interior
		of the contour then we assume that they are 'arbitary' and
		instead define the FOP as an inner polynomial. [KB]
		"""
		polyRoots = scipy.linalg.eig(G1[:p+1,:p+1], G[:p+1,:p+1])[0]+mu
		if np.all([C.contains(z) for z in polyRoots]):
			r, t = r+t+1, 0
			phiZeros.append(polyRoots)

			if verbose: print('Regular poly', r+t, 'roots:', phiZeros[-1])

			# is the number of distinct roots, n=r?
			phiFuncLast = phi(-1)
			for j in range(N-r):
				ip, err = product(lambda z: phiFuncLast(z)*(z-mu)**j, phiFuncLast)

				if verbose: print(j, 'of', N-r, 'err', err, 'abs(ip)', abs(ip))
				if abs(ip) > errStop:
					# n != r so carry on
					if verbose: print('n !=', r)
					break
			else:
				# the for loop did not break
				if verbose: print('n =', r)
				break

		else:
			# define an inner polynomial as phi_{r+t+1} = phi_{t+1} phi_{r}
			t += 1
			phiZeros.append(np.append(phiZeros[t],phiZeros[r]))
			if verbose: print('Inner poly', r+t, 'roots:', phiZeros[-1])

	roots = np.array(phiZeros[-1])

	if verbose:
		print('Computed Roots:')
		print(roots)

	# remove any roots which are not distinct
	rootsToRemove = []
	for i, root in enumerate(roots):
		if len(roots[i+1:]) > 0 and np.any(np.abs(root-roots[i+1:]) < rootTol):
			rootsToRemove.append(i)
	roots = np.delete(roots, rootsToRemove)
	n = len(roots)

	### compute the multiplicities, eq. (1.19) in [KB]
	# V = np.column_stack([roots**i for i in range(n)])
	# if verbose and n > 2: print('Computing ordinary moments')
	# s += [product(lambda z: z**p)[0] for p in range(2, n)]
	# multiplicities = np.dot(s[:n], np.linalg.inv(V))

	### compute the multiplicities, eq. (21) in [SLV]
	V = np.array([[phi(j)(root) for root in roots] for j in range(n)])
	multiplicities = np.dot(np.linalg.inv(V), G[:n,0])

	### The method used in the vandermonde module doesn't seem significantly
	### better than np.dot(s, np.linalg.inv(V)).  Especially since we know
	### the result must be an integer anyway.
	# import vandermonde
	# multiplicities_vandermonde = vandermonde.solve_transpose(np.array(roots), np.array(s))

	### Note that n = rank(H_N) is not used since calculating the
	### rank of a matrix of floats can be quite unstable
	# s_func = lambda p: prod(C, f, df, lambda z: z**p)[0]
	# HN = np.fromfunction(np.vectorize(lambda p,q: s_func(p+q)), shape=(N,N))
	# print('n?', np.linalg.matrix_rank(HN, tol=1e-10))

	if verbose:
		print('Approximations for roots:\n', roots)
		print('Approximations for multiplicities:\n', multiplicities)

	return tuple(roots), tuple(multiplicities)




from __future__ import division
import warnings

import numpy as np
from numpy import inf, pi
import scipy.integrate
import scipy.misc
import numdifftools.fornberg as ndf
import numdifftools

def prod(C, f, df=None, phi=None, psi=None, absTol=1e-12, relTol=1e-12, divMin=3,
	divMax=15, m=2, intMethod='quad', integerTol=inf, verbose=False, callback=None):
	r"""
	Compute the symmetric bilinear form used in (1.12) of [KB]_.

	.. math::

		<\phi,\psi> = \frac{1}{2\pi i} \oint_C \phi(z) \psi(z) \frac{f'(z)}{f(z)} dz.

	Parameters
	----------
	C : :class:`Contour <cxroots.Contour.Contour>`
		A contour in the complex plane for.  No roots or poles of f
		should lie on C.
	f : function
		Function of a single variable f(x)
	df : function, optional
		Function of a single variable, df(x), providing the derivative
		of the function f(x) at the point x.  If not provided then df is
		approximated using a finite difference method.
	phi : function, optional
		Function of a single variable phi(x).  If not provided then
		phi(z)=1.
	psi : function, optional
		Function of a single variable psi(x).  If not provided then
		psi(z)=1.
	absTol : float, optional
		Absolute error tolerance for integration.
	relTol : float, optional
		Relative error tolerance for integration.
	divMin : int, optional
		Only used if intMethod='romb'. Minimum number of divisions before
		the Romberg integration routine is allowed to exit.
	divMax : int, optional
		Only used if intMethod='romb'.  The maximum number of divisions
		before the Romberg integration routine of a path exits.
	m : int, optional
		Only used if df=None and intMethod='quad'.  Must be even.  The
		argument order=m is passed to numdifftools.Derivative and is the
		order of the error term in the Taylor approximation.
	intMethod : {'quad', 'romb'}, optional
		If 'quad' then scipy.integrate.quad is used to perform the
		integral.  If 'romb' then Romberg integraion, using
		scipy.integrate.romb, is performed instead.
	integerTol : float, optional
		Only used when intMethod is 'romb'.  The integration routine will
		not exit unless the result is within integerTol of an integer.
		This is useful when computing the number of roots in a contour,
		which must be an integer.  By default integerTol is inf.
	verbose : bool, optional
		If True runtime information will be printed.  False be default.
	callback : function, optional
		Only used when intMethod is 'romb'.  A function that at each
		step in the iteration is passed the current approximation for
		the integral, the estimated error of that approximation and the
		number of iterations.  If the return of callback evaluates to
		True then the integration will end.

	Returns
	-------
	complex
		The value of the integral <phi, psi>.
	float
		An estimate of the error for the integration.

	References
	----------
	.. [KB] "Computing the zeros of analytic functions" by Peter Kravanja,
		Marc Van Barel, Springer 2000
	"""
	if intMethod == 'romb':
		N = 1
		k = 0
		I = []

		while k < divMax and (len(I) < divMin
			or (abs(I[-2] - I[-1]) > absTol and abs(I[-2] - I[-1]) > relTol*abs(I[-1]))
			or (abs(I[-3] - I[-2]) > absTol and abs(I[-3] - I[-2]) > relTol*abs(I[-2]))
			or abs(int(round(I[-1].real)) - I[-1].real) > integerTol
			or abs(I[-1].imag) > integerTol):
			N = 2*N
			t = np.linspace(0,1,N+1)
			k += 1
			dt = t[1]-t[0]

			integrals = []
			for segment in C.segments:
				# compute/retrieve function evaluations
				fVal = segment.trap_values(f,k)

				if df is None:
					# approximate df/dz with finite difference
					dfdt = np.gradient(fVal, dt)
					dfVal = dfdt/segment.dzdt(t)
				else:
					dfVal = segment.trap_values(df,k)

				segment_integrand = dfVal/fVal*segment.dzdt(t)
				if phi is not None:
					segment_integrand = segment.trap_values(phi,k)*segment_integrand
				if psi is not None:
					segment_integrand = segment.trap_values(psi,k)*segment_integrand

				segment_integral = scipy.integrate.romb(segment_integrand, dx=dt, axis=-1)/(2j*pi)
				integrals.append(segment_integral)

			I.append(sum(integrals))

			if verbose:
				if k > 1:
					print(k, 'I', I[-1], 'err', I[-2] - I[-1])
				else:
					print(k, 'I', I[-1])

			if callback is not None:
				err = abs(I[-2] - I[-1]) if k > 1 else None
				if callback(I[-1], err, k):
					break

		return I[-1], abs(I[-2] - I[-1])

	elif intMethod == 'quad':
		if df is None:
			df = numdifftools.Derivative(f, order=m)
			# df = lambda z: scipy.misc.derivative(f, z, dx=1e-8, n=1, order=3)

			### Too slow
			# ndf.derivative returns an array [f, f', f'', ...]
			# df = np.vectorize(lambda z: ndf.derivative(f, z, n=1)[1])

		I, err = 0, 0
		for segment in C.segments:
			integrand_cache = {}
			def integrand(t):
				if t in integrand_cache.keys():
					i = integrand_cache[t]
				else:
					z = segment(t)
					i = (df(z)/f(z))/(2j*pi) * segment.dzdt(t)
					if phi is not None:
						i = phi(z)*i
					if psi is not None:
						i = psi(z)*i
					integrand_cache[t] = i
				return i

			# integrate real part
			integrand_real = lambda t: np.real(integrand(t))
			result_real = scipy.integrate.quad(integrand_real, 0, 1, full_output=1, epsabs=absTol, epsrel=relTol)
			I_real, abserr_real, infodict_real = result_real[:3]

			# integrate imaginary part
			integrand_imag = lambda t: np.imag(integrand(t))
			result_imag = scipy.integrate.quad(integrand_imag, 0, 1, full_output=1, epsabs=absTol, epsrel=relTol)
			I_imag, abserr_imag, infodict_imag = result_imag[:3]

			I   += I_real + 1j*I_imag
			err += abserr_real + 1j*abserr_imag

		return I, abs(err)

	else:
		raise ValueError("intMethod must be either 'romb' or 'quad'")

class RootError(RuntimeError):
	pass

def count_roots(C, f, df=None, NIntAbsTol=0.07, integerTol=0.1, divMin=3,
	divMax=15, m=2, intMethod='quad', verbose=False):
	r"""
	For a function of one complex variable, f(z), which is analytic in
	and within the contour C, return the number of zeros (counting
	multiplicities) within the contour, N, using Cauchy's argument
	principle,

	.. math::

		N = \frac{1}{2i\pi} \oint_C \frac{f'(z)}{f(z)} dz.

	If df(z), the derivative of f(z), is provided then the above
	integral is computed directly.  Otherwise the derivative is
	approximated using a finite difference method.

	The number of roots is taken to be the closest integer to the
	computed value of the integral and the result is only accepted
	if the integral is within integerTol of the closest integer.

	Parameters
	----------
	C : :class:`Contour <cxroots.Contour.Contour>`
		The contour which encloses the roots of f(z) that are to be
		counted.
	f : function
		Function of a single variable f(z).
	df : function, optional
		Function of a single complex variable, df(z), providing the
		derivative of the function f(z) at the point z.  If not
		provided, df will be approximated using a finite difference
		method.
	NIntAbsTol : float, optional
		Required absolute error tolerance for the contour integration.
		Since the Cauchy integral must be an integer it is only
		necessary to distinguish which integer the integral is
		converging towards.  Therefore, NIntAbsTol can be fairly large.
	integerTol : float, optional
		The evaluation of the Cauchy integral will be accepted if its
		value is within integerTol of the closest integer.
	divMin : int, optional
		Only used if intMethod='romb'. Minimum number of divisions
		before the Romberg integration routine is allowed to exit.
	divMax : int, optional
		Only used if intMethod='romb'.  The maximum number of divisions
		before the Romberg integration routine of a path exits.
	m : int, optional
		Only used if df=None and intMethod='quad'.  The argument order=m
		is passed to numdifftools.Derivative and is the order of the
		error term in the Taylor approximation.  m must be even.
	intMethod : {'quad', 'romb'}, optional
		If 'quad' then scipy.integrate.quad is used to perform the
		integral.  If 'romb' then Romberg integraion, using
		scipy.integrate.romb, is performed instead.
	verbose : bool, optional
		If True certain messages regarding the integration will be
		printed.

	Returns
	-------
	int
		The number of zeros of f (counting multiplicities) which lie
		within the contour C.
	"""
	if verbose:
		print('Computing number of roots within', C)

	with warnings.catch_warnings():
		# ignore warnings and catch if I is NaN later
		warnings.simplefilter("ignore")
		I, err = prod(C, f, df, absTol=NIntAbsTol, relTol=0, divMin=divMin,
			divMax=divMax, m=m, intMethod=intMethod, verbose=verbose, integerTol=integerTol)

	if intMethod == 'romb':
		C._numberOfDivisionsForN = int(np.log2(len(C.segments[0]._trapValuesCache[f])-1))

	if np.isnan(I):
		raise RootError("""Result of integral is an invalid value.
						   Most likely because of a divide by zero error.""")

	elif abs(int(round(I.real)) - I.real) < integerTol and abs(I.imag) < integerTol:
		# integral is sufficiently close to an integer
		numberOfZeros = int(round(I.real))
		return numberOfZeros

	else:
		raise RootError("The number of enclosed roots has not converged to an integer")




from __future__ import division
from numpy import inf

def iterateToRoot(x0, f, df=None, steptol=1e-12, roottol=1e-12, maxIter=20, 
	attemptBest=False, verbose=False, callback=None):
	"""
	Starting with initial point x0 iterate to a root of f. This function 
	is called during the rootfinding process to refine any roots found.
	If df is given then the Newton-Raphson method, :func:`~cxroots.IterativeMethods.newton`,
	will be used, otherwise Muller's method, :func:`~cxroots.IterativeMethods.muller`, will be used instead.

	Parameters
	----------
	x0 : complex
		An initial point for the iteration.
	f : function
		Function of a single variable which we seek to find a root of.
	df : function, optional
		The derivative of f.
	steptol: float, optional
		The routine ends if the step size, dx, between sucessive
		iterations satisfies abs(dx) < steptol and attemptBest is False.
	roottol: float, optional
		The routine ends if abs(f(x)) < roottol and attemptBest is False.
	maxIter : int, optional
		The routine ends after maxIter iterations.
	attemptBest : bool, optional
		If True then routine ends if the error of the previous iteration, 
		x0, was at least as good as the current iteration, x, in the 
		sense that abs(f(x)) >= abs(f(x0)) and the previous iteration 
		satisfied either abs(dx0) < steptol or abs(f(x0)) < roottol.  In 
		this case the previous iteration is returned as the approximation 
		of the root.
	verbose : bool, optional
		Print x, dx and f(x) at each step of the iteration.
	callback : function, optional
		After each iteration callback(x, dx, f(x), iteration) will be 
		called where 'x' is the current iteration of the estimated root, 
		'dx' is the step size between the previous and current 'x' and 
		'iteration' the number of iterations that have been taken.  If 
		the callback function evaluates to True then the routine will end.

	Returns
	-------
	complex
		An approximation for a root of f.  If the rootfinding was 
		unsucessful then None will be returned instead.
	"""
	if verbose: print('Refining root:', x0)

	if df is not None:
		try:
			root, err = newton(x0, f, df, steptol, 0, maxIter, attemptBest, verbose, callback)
		except (RuntimeError, OverflowError):
			return None
	else:
		# Muller's method:
		f_muller = lambda z: complex(f(z))
		x1, x2, x3 = x0, x0*(1 + 1e-8) + 1e-8, x0*(1 - 1e-8) - 1e-8
		root, err = muller(x1, x2, x3, f_muller, steptol, 0, maxIter, attemptBest, verbose, callback)

	if err < roottol:
		return root

def muller(x1, x2, x3, f, steptol=1e-12, roottol=1e-12, maxIter=20, 
	attemptBest=False, verbose=False, callback=None):
	"""
	A wrapper for mpmath's implementation of Muller's method.  

	Parameters
	----------
	x1 : float or complex
		An initial point for iteration, should be close to a root of f.
	x2 : float or complex
		An initial point for iteration, should be close to a root of f.  
		Should not equal x1.
	x3 : float or complex
		An initial point for iteration, should be close to a root of f.  
		Should not equal x1 or x2.
	f : function
		Function of a single variable which we seek to find a root of.
	steptol: float, optional
		The routine ends if the step size, dx, between sucessive
		iterations satisfies abs(dx) < steptol and attemptBest is False.
	roottol: float, optional
		The routine ends if abs(f(x)) < roottol and attemptBest is False.
	maxIter : int, optional
		The routine ends after maxIter iterations.
	attemptBest : bool, optional
		If True then routine ends if the error of the previous iteration, 
		x0, was at least as good as the current iteration, x, in the 
		sense that abs(f(x)) >= abs(f(x0)) and the previous iteration 
		satisfied either abs(dx0) < steptol or abs(f(x0)) < roottol.  In 
		this case the previous iteration is returned as the approximation 
		of the root.
	verbose : bool, optional
		Print x, dx and f(x) at each step of the iteration.
	callback : function, optional
		After each iteration callback(x, dx, f(x), iteration) will be 
		called where 'x' is the current iteration of the estimated root, 
		'dx' is the step size between the previous and current 'x' and 
		'iteration' the number of iterations that have been taken.  If 
		the callback function evaluates to True then the routine will end.

	Returns
	-------
	complex
		The approximation to a root of f.
	float
		abs(f(x)) where x is the final approximation for the root of f.
	"""
	from mpmath import mp, mpmathify
	from mpmath.calculus.optimization import Muller

	# mpmath insists on functions accepting mpc
	f_mpmath = lambda z: mpmathify(f(complex(z)))

	mull = Muller(mp, f_mpmath, (x1, x2, x3), verbose=False)
	iteration = 0
	x0 = x3

	x, err = x0, abs(f(x0))
	err0, dx0 = inf, inf
	try:
		for x, dx in mull:
			err = abs(f_mpmath(x))

			if verbose: print(iteration, 'x', x, '|f(x)|', err, 'dx', dx)

			if callback is not None and callback(x, dx, err, iteration+1):
				break

			if not attemptBest and (abs(dx) < steptol or err < roottol) or iteration > maxIter:
				break

			if attemptBest and (abs(dx0) < steptol or err0 < roottol) and err >= err0:
				# The previous iteration was a better appproximation the current one so  
				# assume that that was as close to the root as we are going to get.
				x, err = x0, err0
				break

			iteration += 1
			x0 = x

			if attemptBest:
				# record previous error for comparison
				dx0, err0 = dx, err

	except ZeroDivisionError:
		# ZeroDivisionError comes up if the error is evaluated to be zero
		pass

	if verbose: print('Final approximation: x=', complex(x), '|f(x)|=', float(err))

	# cast mpc and mpf back to regular complex and float
	return complex(x), float(err)


def newton(x0, f, df, steptol=1e-12, roottol=1e-12, maxIter=20, 
	attemptBest=False, verbose=False, callback=None):
	"""
	Find an approximation to a point xf such that f(xf)=0 for a 
	scalar function f using Newton-Raphson iteration starting at 
	the point x0.

	Parameters
	----------
	x0 : float or complex
		Initial point for Newton iteration, should be as close as
		possible to a root of f
	f : function
		Function of a single variable which we seek to find a root of.
	df : function
		Function of a single variable, df(x), providing the
		derivative of the function f(x) at the point x
	steptol: float, optional
		The routine ends if the step size, dx, between sucessive
		iterations satisfies abs(dx) < steptol and attemptBest is False.
	roottol: float, optional
		The routine ends if abs(f(x)) < roottol and attemptBest is False.
	maxIter : int, optional
		The routine ends after maxIter iterations.
	attemptBest : bool, optional
		If True then routine ends if the error of the previous iteration, 
		x0, was at least as good as the current iteration, x, in the 
		sense that abs(f(x)) >= abs(f(x0)) and the previous iteration 
		satisfied either abs(dx0) < steptol or abs(f(x0)) < roottol.  In 
		this case the previous iteration is returned as the approximation 
		of the root.
	verbose : bool, optional
		Print x, dx and f(x) at each step of the iteration.
	callback : function, optional
		After each iteration callback(x, dx, f(x), iteration) will be 
		called where 'x' is the current iteration of the estimated root, 
		'dx' is the step size between the previous and current 'x' and 
		'iteration' the number of iterations that have been taken.  If 
		the callback function evaluates to True then the routine will end.

	Returns
	-------
	complex
		The approximation to a root of f.
	float
		abs(f(x)) where x is the final approximation for the root of f.
	"""	
	x, y = x0, f(x0)
	dx0, y0 = inf, y
	for iteration in range(maxIter):
		dx = -y/df(x)
		x += dx
		y  = f(x)

		if verbose: print('x', x, 'f(x)', y, 'dx', dx)

		if callback is not None and callback(x, dx, y, iteration+1):
			break

		if not attemptBest and (abs(dx) < steptol or abs(y) < roottol):
			break

		if attemptBest and (abs(dx0) < steptol or abs(y0) < roottol) and abs(y) > abs(y0):
			break

		if attemptBest:
			# store previous dx and y
			dx0, y0 = dx, y

	if verbose: print('Final approximation: x=', x, '|f(x)|=', abs(y))

	return x, abs(y)

def secant(x1, x2, f, steptol=1e-12, roottol=1e-12, maxIter=30, callback=None):
	"""
	Find an approximation to a point xf such that f(xf)=0 for a 
	scalar function f using the secant method.  The method requires
	two initial points x1 and x2, ideally close to a root,
	and proceeds iteratively.

	Parameters
	----------
	x1 : float or complex
		An initial point for iteration, should be close to a 
		root of f.
	x2 : float or complex
		An initial point for iteration, should be close to a 
		root of f.  Should not equal x1.
	f : function
		Function of a single variable which we seek to find a root of.
	steptol: float, optional
		The routine ends if the step size, dx, between sucessive
		iterations satisfies abs(dx) < steptol and attemptBest is False.
	roottol: float, optional
		The routine ends if abs(f(x)) < roottol and attemptBest is False.
	maxIter : int, optional
		The routine ends after maxIter iterations.
	callback : function, optional
		After each iteration callback(x, dx, f(x), iteration) will be 
		called where 'x' is the current iteration of the estimated root, 
		'dx' is the step size between the previous and current 'x' and 
		'iteration' the number of iterations that have been taken.  If 
		the callback function evaluates to True then the routine will end.

	Returns
	-------
	complex
		The approximation to a root of f.
	float
		abs(f(x)) where x is the final approximation for the root of f.
	"""
	# As in "Numerical Recipies 3rd Edition" pick the bound with the 
	# smallest function value as the most recent guess
	y1, y2 = f(x1), f(x2)
	if abs(y1) < abs(y2):
		x1, x2 = x2, x1
		y1, y2 = y2, y1

	for iteration in range(maxIter):
		dx =  -(x2-x1)*y2/(y2-y1)
		x1, x2 = x2, x2 + dx
		y1, y2 = y2, f(x2)

		if callback is not None and callback(x2, dx, y2, iteration+1):
			break

		if abs(dx) < steptol or abs(y2) < roottol:
			break

	return x2, abs(y2)



from __future__ import division

import numpy as np
import functools

from .CountRoots import count_roots
from .ApproximateRoots import approximate_roots
from .RootFinder import find_roots
from .DemoRootFinder import demo_find_roots
from .Misc import remove_para
from .Paths import ComplexPath


class Contour(object):
	"""
	A base class for contours in the complex plane.

	Attributes
	----------
	centralPoint : complex
		The point at the center of the contour.
	area : float
		The surface area of the contour.
	"""
	def __init__(self, segments):
		self.segments = np.array(segments, dtype=object)

	def __call__(self, t):
		r"""
		The point on the contour corresponding the value of the
		parameter t.

		Parameters
		----------
		t : float
			A real number :math:`0\leq t \leq 1` which parameterises
			the contour.

		Returns
		-------
		complex
			A point on the contour.

		Example
		-------

		>>> from cxroots.Paths import Circle
		>>> c = Circle(0,1)	# Circle |z|=1 parameterised by e^{it}
		>>> c(0.25)
		(6.123233995736766e-17+1j)
		>>> c(0) == c(1)
		True
		"""
		t = np.array(t)
		N = len(self.segments)
		segmentIndex = np.array(N*t, dtype=int)
		segmentIndex = np.mod(segmentIndex, N)

		if hasattr(segmentIndex, '__iter__'):
			return np.array([self.segments[i](N*t[ti]%1) for ti, i in enumerate(segmentIndex)])
		else:
			return self.segments[segmentIndex](N*t%1)

	@property
	def centralPoint(self):
		raise NotImplementedError('centralPoint needs to be implemented in the subclass.')

	@property
	def area(self):
		raise NotImplementedError('area needs to be implemented in the subclass.')

	def contains(self, z):
		"""
		Tests whether the point z is within the contour.

		Parameters
		----------
		z : complex

		Returns
		-------
		bool
			True if z lies within the contour and false otherwise.
		"""
		raise NotImplementedError('contains() needs to be implemented in the subclass.')

	@functools.wraps(ComplexPath.plot)
	def plot(self, *args, **kwargs):
		self._sizePlot()
		for segment in self.segments:
			segment.plot(*args, **kwargs)

	def _sizePlot(self):
		import matplotlib.pyplot as plt
		t = np.linspace(0,1,1000)
		z = self(t)
		xpad = (max(np.real(z))-min(np.real(z)))*0.1
		ypad = (max(np.imag(z))-min(np.imag(z)))*0.1

		xmin = min(np.real(z))-xpad
		xmax = max(np.real(z))+xpad
		ymin = min(np.imag(z))-ypad
		ymax = max(np.imag(z))+ypad
		plt.xlim([xmin, xmax])
		plt.ylim([ymin, ymax])

	def show(self, saveFile=None, **plotKwargs):
		"""
		Shows the contour as a 2D plot in the complex plane.  Requires
		Matplotlib.

		Parameters
		----------
		saveFile : str (optional)
			If given then the plot will be saved to disk with name
			'saveFile'.  If saveFile=None the plot is shown on-screen.
		**plotKwargs
			Key word arguments are as in :meth:`~cxroots.Contour.Contour.plot`.
		"""
		import matplotlib.pyplot as plt
		self.plot(**plotKwargs)

		if saveFile is not None:
			plt.savefig(saveFile, bbox_inches='tight')
			plt.close()
		else:
			plt.show()

	def subdivisions(self, axis='alternating'):
		"""
		A generator for possible subdivisions of the contour.

		Parameters
		----------
		axis : str, 'alternating' or any element of self.axisName.
			The axis along which the line subdividing the contour is a
			constant (eg. subdividing a circle along the radial axis
			will give an outer annulus and an inner circle).  If
			alternating then the dividing axis will always be different
			to the dividing axis used to create the contour which is now
			being divided.

		Yields
		------
		tuple
			A tuple with two contours which subdivide the original
			contour.
		"""
		if axis == 'alternating':
			if hasattr(self,'_createdBySubdivisionAxis'):
				axis = (self._createdBySubdivisionAxis + 1)%len(self.axisName)
			else:
				axis = 0

		for divisionFactor in divisionFactorGen():
			yield self.subdivide(axis, divisionFactor)

	def distance(self, z):
		"""
		Get the distance from the point z in the complex plane to the
		nearest point on the contour.

		Parameters
		----------
		z : complex
			The point from which to measure the distance to the closest
			point on the contour to z.

		Returns
		-------
		float
			The distance from z to the point on the contour which is
			closest to z.
		"""
		return min(segment.distance(z) for segment in self.segments)

	@functools.wraps(ComplexPath.integrate)
	def integrate(self, f, **integrationKwargs):
		return sum([segment.integrate(f, **integrationKwargs) for segment in self.segments])

	@remove_para('C')
	@functools.wraps(count_roots)
	def count_roots(self, f, df=None, **kwargs):
		return count_roots(self, f, df, **kwargs)

	@remove_para('C')
	@functools.wraps(approximate_roots)
	def approximate_roots(self, N, f, df=None, **kwargs):
		return approximate_roots(self, N, f, df, **kwargs)

	@remove_para('originalContour')
	@functools.wraps(find_roots)
	def roots(self, f, df=None, **kwargs):
		return find_roots(self, f, df, **kwargs)

	@remove_para('C')
	@functools.wraps(demo_find_roots)
	def demo_roots(self, *args, **kwargs):
		return demo_find_roots(self, *args, **kwargs)

def divisionFactorGen():
	"""A generator for divisionFactors."""
	yield 0.3	# being off-center is a better first choice for certain problems

	x = 0.5
	yield x
	for diff in np.linspace(0, 0.5, int(1+10/2.))[1:-1]:
		yield x + diff
		yield x - diff



import pytest

from cxroots import AnnulusSector
from cxroots import Rectangle

def test_PolarRect_contains():
	r0=8.938
	r1=9.625
	phi0=6.126
	phi1=6.519

	z = (9-1.04825594683e-18j)
	C = AnnulusSector(0, [r0,r1], [phi0,phi1])

	# import matplotlib.pyplot as plt
	# plt.scatter((9),(0))
	# C.show()

	assert C.contains(z)


def test_rect_contains():
	C = Rectangle([-2355, -1860], [-8810, -8616])

	assert C.contains(-2258-8694j)
	assert not C.contains(-2258-8500j)



import docrep
docstrings = docrep.DocstringProcessor()

def doc_tab_to_space(func):
	"""
	docrep doesn't like tabs
	"""
	func.__doc__ = func.__doc__.replace('\t', '    ')
	return func

def remove_para(*paras):
	def wrapper(func):
		func = doc_tab_to_space(func)
		func = docstrings.dedent(func)
		func.__doc__ = docstrings.delete_params_s(func.__doc__, paras)
		return func
	return wrapper

class NumberOfRootsChanged(Exception):
    pass



import pytest
import numpy as np
from scipy import cos, sin

from cxroots import Rectangle

@pytest.mark.parametrize('useDerivative', [True, False])
def test_count_roots(useDerivative):
	"""
	Example from "Locating all the Zeros of an Analytic Function in one Complex Variable"
	M.Dellnitz, O.Schutze, Q.Zheng, J. Compu. and App. Math. (2002), Vol.138, Issue 2

	There should be 424 roots inside this contour
	"""
	C  = Rectangle([-20.3,20.7], [-20.3,20.7])
	f  = lambda z: z**50 + z**12 - 5*sin(20*z)*cos(12*z) - 1
	df = lambda z: 50*z**49 + 12*z**11 + 60*sin(12*z)*sin(20*z) - 100*cos(12*z)*cos(20*z)

	if not useDerivative: df = None

	assert C.count_roots(f, df, verbose=True) == 424


@pytest.mark.xfail(reason='Possibly lack of high precision arithmetic')
def test_RingOscillator():
	# problem in section 4.3 of [DSZ]
	from cxroots import Rectangle, findRoots

	def A(z):
		t = 2.
		return np.array([[-0.0166689-2.12e-14*z, 1/60. + 6e-16*exp(-t*z)*z],
		 		  		 [0.0166659+exp(-t*z)*(-0.000037485+6e-16*z), -0.0166667-6e-16*z]])

	def dA(z):
		t = 2.
		return np.array([[-2.12e-14*np.ones_like(z), 6e-16*exp(-t*z) - t*6e-16*exp(-t*z)*z],
				   		 [-t*exp(-t*z)*(-0.000037485+6e-16*z)+exp(-t*z)*6e-16, -6e-16*np.ones_like(z)]])
	
	def f(z):
		AVal = np.rollaxis(A(z),-1,0)
		return np.linalg.det(AVal)

	def df(z):
		AVal  = A(z)
		dAVal = dA(z)
		return dAVal[0,0]*AVal[1,1] + AVal[0,0]*dAVal[1,1] - dAVal[0,1]*AVal[1,0] - AVal[0,1]*dAVal[1,0]


	box = Rectangle([-12,0], [-40,40])
	# roots_fdf = findRoots(box, f, df)
	# roots_f = findRoots(box, f)

	# XXX: No roots are recorded within the initial contour.
	# 	Perhaps because the coefficents of z are very small?
	# 	Perhaps need higher precision?
	assert box.count_enclosed_roots(f, df) != 0
	assert box.count_enclosed_roots(f) != 0

	# compare with fig 4 of [DSZ]
	# roots_fdf.show()



from __future__ import division
import numpy as np
from scipy import pi

from ..Contour import Contour
from ..Paths import ComplexArc
from .AnnulusSector import AnnulusSector

class Annulus(Contour):
	"""
	An annulus in the complex plane with the outer circle positively oriented
	and the inner circle negatively oriented.

	Parameters
	----------
	center : complex
		The center of the annulus in the complex plane.
	radii : tuple
		A tuple of length two of the form (inner_radius, outer_radius).

	Examples
	--------
	.. plot::
		:include-source:

		from cxroots import Annulus
		annulus = Annulus(center=0, radii=(0.5,0.75))
		annulus.show()
	"""
	def __init__(self, center, radii):
		self.center = center
		self.radii = radii
		self.axisName = ('r', 'phi')

		segments = [ComplexArc(center, radii[1], 0, 2*pi), ComplexArc(center, radii[0], 0, -2*pi)]
		super(Annulus, self).__init__(segments)

	def __str__(self):
		return 'Annulus: center={center.real:.3f}{center.imag:+.3f}i, inner radius={radii[0]:.3f}, outer radius={radii[1]:.3f}'.format(center=self.center, radii=self.radii)

	@property
	def centralPoint(self):
		# get the central point within the contour
		r = (self.radii[0] + self.radii[1])/2
		return r

	@property
	def area(self):
		return pi*(self.radii[1]**2 - self.radii[0]**2)

	def contains(self, z):
		""" Returns True if the point z lies within the contour, False if otherwise """
		return self.radii[0] < abs(z - self.center) < self.radii[1]

	def subdivide(self, axis, divisionFactor=0.5):
		"""
		Subdivide the contour

		Parameters
		----------
		axis : str, can be either 'r' or 'phi'
			The axis along which the line subdividing the contour is a constant.
		divisionFactor : float in range (0,1), optional
			Determines the point along 'axis' at which the line dividing the box is placed

		Returns
		-------
		boxes : list of contours
			Two annuluses if axis is 'r'.
			Two half-annuluses oriented according to divisionFactor if axis is 'phi'.
		"""
		if axis == 'r' or self.axisName[axis] == 'r':
			midpoint = self.radii[0] + divisionFactor*(self.radii[1]-self.radii[0])
			box1 = Annulus(self.center, [self.radii[0], midpoint])
			box2 = Annulus(self.center, [midpoint, self.radii[1]])

			box1.segments[1] = self.segments[1]
			box2.segments[0] = self.segments[0]
			box1.segments[0]._reversePath = box2.segments[1]
			box2.segments[1]._reversePath = box1.segments[0]

			box1._createdBySubdivisionAxis = axis
			box2._createdBySubdivisionAxis = axis

		elif axis == 'phi' or self.axisName[axis] == 'phi':
			# Subdividing into two radial boxes rather than one to
			# ensure that an error is raised if one of the new paths
			# is too close to a root
			# XXX: introduce another parameter for phi1

			phi0 = 2*pi*divisionFactor
			phi1 = phi0 + pi

			box1 = AnnulusSector(self.center, self.radii, [phi0, phi1])
			box2 = AnnulusSector(self.center, self.radii, [phi1, phi0])

			box1.segments[0]._reversePath = box2.segments[2]
			box2.segments[2]._reversePath = box1.segments[0]
			box1.segments[2]._reversePath = box2.segments[0]
			box2.segments[0]._reversePath = box1.segments[2]

		for box in [box1, box2]:
			box._createdBySubdivisionAxis = axis
			box._parentBox = self
		self._childBoxes = [box1, box2]

		return box1, box2

	def randomPoint(self):
		""" Returns a random point inside the Annulus """
		r   = np.random.uniform(*self.radii)
		phi = np.random.uniform(0,2*pi)
		return r*exp(1j*phi) + self.center



from __future__ import division

import numpy as np

from .RootFinder import find_roots_gen

def demo_find_roots(originalContour, f, df=None, saveFile=None, automaticAnim=False,
	returnAnim=False, writer=None, **rootsKwargs):
	"""
	An animated demonstration of the root finding process using matplotlib.

	Parameters
	----------
	saveFile : str, optional
		If given then the animation will be saved to disk with filename
		equal to saveFile instead of being shown.
	automaticAnim : bool, optional
		If False (default) then press SPACE to step the animation forward
		If True then the animation will play automatically until all the
		roots have been found.
	returnAnim : bool, optional
		If True then the matplotlib animation object will be returned
		instead of being shown.  Defaults to False.
	writer : str, optional
		Passed to :meth:`matplotlib.animation.FuncAnimation.save`.
	**rootsKwargs
		Additional key word arguments passed to :meth:`~cxroots.Contour.Contour.roots`.
	"""
	import matplotlib.pyplot as plt
	from matplotlib import animation
	fig = plt.gcf()
	ax = plt.gca()

	rootFinder = find_roots_gen(originalContour, f, df, **rootsKwargs)

	def init():
		originalContour.plot(linecolor='k', linestyle='--')
		originalContour._sizePlot()

	def update_frame(args):
		roots, multiplicities, boxes, numberOfRemainingRoots = args

		plt.cla() # clear axis
		originalContour.plot(linecolor='k', linestyle='--')
		for box in boxes:
			if not hasattr(box, '_color'):
				cmap = plt.get_cmap('jet')
				box._color = cmap(np.random.random())

			plt.text(box.centralPoint.real, box.centralPoint.imag, box._numberOfRoots)
			box.plot(linecolor=box._color)

		plt.scatter(np.real(roots), np.imag(roots), color='k', marker='x')
		ax.text(0.02, 0.95, 'Zeros remaining: %i'%numberOfRemainingRoots, transform=ax.transAxes)
		originalContour._sizePlot()
		fig.canvas.draw()

	if saveFile:
		automaticAnim = True

	if automaticAnim or returnAnim:
		anim = animation.FuncAnimation(fig, update_frame, init_func=init, frames=rootFinder)
		if returnAnim: return anim

	else:
		def draw_next(event):
			if event.key == ' ':
				update_frame(next(rootFinder))
		fig.canvas.mpl_connect('key_press_event', draw_next)

	if saveFile:
		anim.save(filename=saveFile, fps=1, dpi=200, writer=writer)
		plt.close()
	else:
		plt.show()



import pytest

from cxroots.IterativeMethods import secant
from scipy import pi, cos, sin

def test_secant():
	# example from Table 2.5 of "Numerical Analysis" by Richard L. Burden, J. Douglas Faires
	f  = lambda x: cos(x)-x
	df = lambda x: -sin(x)-1

	iterations = []
	callback = lambda x, dx, y, iteration: iterations.append(x)
	x, err = secant(0.5, pi/4, f, callback=callback)
	iterations.append(x)
	
	correct_iterations = [0.7363841388,
		  				  0.7390581392,
		  				  0.7390851493,
		  				  0.7390851332,
		  				  0.7390851332]

	assert iterations == pytest.approx(correct_iterations)



from __future__ import division
import warnings
import functools

import numpy as np

from .IterativeMethods import iterateToRoot
from .CountRoots import RootError
from .RootResult import RootResult
from .Misc import doc_tab_to_space, docstrings, NumberOfRootsChanged

class MultiplicityError(RuntimeError):
	pass

class countCalls:
	"""
	Count how many times a given function is called.
	"""
	def __init__(self, func):
		self.func = func
		self.calls = 0
		self.points = 0

	def __call__(self, z):
		self.calls += 1
		if hasattr(z, '__len__'):
			self.points += len(z)
		else:
			self.points += 1
		return self.func(z)

@docstrings.get_sectionsf('find_roots_gen')
@docstrings.dedent
@doc_tab_to_space
def find_roots_gen(originalContour, f, df=None, guessRoots=[], guessRootSymmetry=None,
	newtonStepTol=1e-14, attemptIterBest=True, newtonMaxIter=50, rootErrTol=1e-10,
	absTol=0, relTol=1e-12, integerTol=0.1, NIntAbsTol=0.07, M=5, errStop=1e-10,
	intMethod='quad', divMin=3, divMax=15, m=2, verbose=False):
	"""
	A generator which at each step takes a contour and either finds all
	the zeros of f within it or subdivides it further.  Based on the
	algorithm in [KB]_.

	Parameters
	----------
	originalContour : :class:`Contour <cxroots.Contour.Contour>`
		The contour which bounds the region in which all the roots of
		f(z) are sought.
	f : function
		A function of a single complex variable, z, which is analytic
		within the contour and has no poles or roots on the contour.
	df : function, optional
		A function of a single complex variable which is the derivative
		of the function f(z). If df is not given then it will be
		approximated with a finite difference formula.
	guessRoots : list, optional
		A list of known roots or guesses for roots (they are checked
		before being accepted).
	guessRootSymmetry : function, optional
		A function of a single complex variable, z, which returns a list
		of all points which are expected to be roots of f, given that z
		is a root of f.
	newtonStepTol : float, optional
		The required accuracy of the root.  The iterative method used to
		give a final value for each root will exit if the step size, dx,
		between sucessive iterations satisfies abs(dx) < newtonStepTol
		and iterBestAttempt is False.
	attemptIterBest : bool, optional
		If True then the iterative method used to refine the roots will
		exit when error of the previous iteration, x0, was at least as
		good as the current iteration, x, in the sense that
		abs(f(x)) >= abs(f(x0)) and the previous iteration satisfied
		abs(dx0) < newtonStepTol.  In this case the preivous iteration
		is returned as the approximation of the root.
	newtonMaxIter : int, optional
		The iterative method used to give a final value for each root
		will exit if the number of iterations exceeds newtonMaxIter.
	rootErrTol : float, optional
		A complex value z is considered a root if abs(f(z)) < rootErrTol
	absTol : float, optional
		Absolute error tolerance used by the contour integration.
	relTol : float, optional
		Relative error tolerance used by the contour integration.
	integerTol : float, optional
		A number is considered an integer if it is within integerTol of
		an integer.  Used when determing if the value for the number of
		roots within a contour and the values of the computed
		multiplicities of roots are acceptably close to integers.
	NIntAbsTol : float, optional
		The absolute error tolerance used for the contour integration
		when determining the number of roots within a contour.  Since
		the result of this integration must be an integer it can be much
		less accurate than usual.
	M : int, optional
		If the number of roots (including multiplicites) within a
		contour is greater than M then the contour is subdivided
		further.  M must be greater than or equal to the largest
		multiplcity of any root.
	errStop : float, optional
		The number of distinct roots within a contour, n, is determined
		by checking if all the elements of a list of contour integrals
		involving formal orthogonal polynomials are sufficently close to
		zero, ie. that the absolute value of each element is < errStop.
		If errStop is too large/small then n may be smaller/larger than
		it actually is.
	intMethod : {'quad', 'romb'}, optional
		If 'quad' then :func:`scipy.integrate.quad` is used to perform the
		integral.  If 'romb' then Romberg integraion, using
		:func:`scipy.integrate.romb`, is performed instead.  Typically, quad is
		the better choice but it requires that the real and imaginary
		parts of each integral are calculated sepeartely, in addition,
		if df is not provided, 'quad' will require additional function
		evaluations to approximate df at each point that f is evaluated
		at.  If evaluating f is expensive then 'romb' may be more
		efficient since it computes the real and imaginary parts
		simultaniously and if df is not provided it will approximate it
		using only the values of f that would be required by the
		integration routine in any case.
	divMin : int, optional
		If the Romberg integration method is used then divMin is the
		minimum number of divisions before the Romberg integration
		routine is allowed to exit.
	divMax : int, optional
		If the Romberg integration method is used then divMax is the
		maximum number of divisions before the Romberg integration
		routine exits.
	m : int, optional
		Only used if df=None and method='quad'.  The argument order=m is
		passed to :func:`numdifftools.Derivative` and is the order of the error
		term in the Taylor approximation.  m must be even.
	verbose : bool, optional
		If True certain messages concerning the rootfinding process will
		be printed.

	Yields
	------
	list
		Roots of f(z) within the contour originalContour
	list
		Multiplicites of roots
	deque
		The contours which still contain roots
	int
		Remaining number of roots to be found within the contour

	References
	----------
	.. [KB] Peter Kravanja, Marc Van Barel, "Computing the Zeros of
		Anayltic Functions", Springer (2000)
	"""
	from .contours.Circle import Circle

	# wrap f to record the number of function calls
	f = countCalls(f)

	countKwargs = {'f':f, 'df':df, 'NIntAbsTol':NIntAbsTol, 'integerTol':integerTol,
		'divMin':divMin, 'divMax':divMax, 'm':m, 'intMethod':intMethod, 'verbose':verbose}

	try:
		# compute the total number of zeros, including multiplicities, within the originally given contour
		originalContour._numberOfRoots = originalContour.count_roots(**countKwargs)
	except RuntimeError:
		raise RuntimeError("""
			Integration along the initial contour has failed.
			There is likely a root on or close to the initial contour.
			Try changing the initial contour, if possible.""")

	if verbose:
		print('Total number of roots (counting multiplicities) within the original contour =', originalContour._numberOfRoots)

	roots = []
	multiplicities = []
	failedContours = []
	contours = []
	contours.append(originalContour)

	def subdivide(parentContour):
		"""Given a contour, parentContour, subdivide it into multiple contours."""
		if verbose:
			print('Subdividing', parentContour)

		numberOfRoots = None
		for subcontours in parentContour.subdivisions():
			# if a contour has already been used and caused an error then skip it
			failedContourDesc = list(map(str, failedContours))
			if np.any([contourDesc in failedContourDesc for contourDesc in list(map(str, subcontours))]):
				continue

			# if a contour is near to a known root then skip it
			for root in roots:
				if np.any(np.abs([contour.distance(root) for contour in subcontours]) < 0.01):
					continue

			try:
				numberOfRoots = [contour.count_roots(**countKwargs) for contour in np.array(subcontours)]
				while parentContour._numberOfRoots != sum(numberOfRoots):
					if verbose:
						print('Number of roots in sub contours not adding up to parent contour.')
						print('Recomputing number of roots in parent and child contours with NIntAbsTol = ', .5*NIntAbsTol)

					tempCountKwargs = countKwargs.copy()
					tempCountKwargs['NIntAbsTol'] *= 0.5
					parentContour._numberOfRoots = parentContour.count_roots(**tempCountKwargs)
					numberOfRoots = [contour.count_roots(**tempCountKwargs) for contour in np.array(subcontours)]

				if parentContour._numberOfRoots == sum(numberOfRoots):
					break

			except RootError:
				# If the number of zeros within either of the new contours is not an integer then it is
				# likely that the introduced line which subdivides 'parentContour' lies on a zero.
				# To avoid this we will try to place the subdividing line at a different point along
				# the division axis
				if verbose:
					print('RootError encountered when subdivding', parentContour, 'into:')
					print(subcontours[0])
					print(subcontours[1])
				continue

		if numberOfRoots is None or parentContour._numberOfRoots != sum(numberOfRoots):
			# The list of subdivisions has been exhaused and still the number of enclosed zeros does not add up
			raise RuntimeError("""Unable to subdivide contour:
				\t%s
				""" % parentContour)

		# record number of roots within each sub-contour
		for i, contour in enumerate(subcontours):
			contour._numberOfRoots = numberOfRoots[i]

		# add these sub-contours to the list of contours to find the roots in
		contours.extend([contour for i, contour in enumerate(subcontours) if numberOfRoots[i] != 0])


	def remove_siblings_children(C):
		"""
		Remove the contour C and all its siblings and children from the
		list of contours to be examined/subdivided.
		"""
		try:
			contours.remove(C)
		except ValueError:
			pass

		# get sibling and child contours
		# siblings:
		relations = C._parentContour._childcontours
		relations.remove(C)

		# children:
		if hasattr(C, '_childcontours'):
			relations.extend(C._childcontours)

		# interate over all relations
		for relation in relations:
			remove_relations(relation)


	def addRoot(root, multiplicity):
		# check that the root we have found is distinct from the ones we already have
		if not roots or np.all(abs(np.array(roots) - root) > newtonStepTol):
			# add the root to the list if it is within the original contour
			if originalContour.contains(root):
				roots.append(root)
				multiplicities.append(multiplicity)
				if verbose: print('Recorded root', root, 'multiplicity', multiplicity)
			elif verbose: print('Root', root, 'ignored as not within original contour.')

			# check to see if there are any other roots implied by the given symmetry
			if guessRootSymmetry is not None:
				for x0 in guessRootSymmetry(root):
					# first check that x0 is distinct from the roots we already have
					if np.all(abs(np.array(roots) - x0) > newtonStepTol):
						if verbose:
							print(root, 'is a root so guessRootSymmetry suggests that', x0, 'might also be a root.')
						root = iterateToRoot(x0, f, df, newtonStepTol, rootErrTol, newtonMaxIter, attemptIterBest, verbose)
						if root is not None:
							contours.append(Circle(root, 1e-3))
							contours[-1]._numberOfRoots = contours[-1].count_roots(**countKwargs)

		elif verbose:
			print('Already recorded root', root)

	# Add contours surrounding known roots so that they will be checked
	for root in guessRoots:
		contours.append(Circle(root, 1e-3))
		contours[-1]._numberOfRoots = contours[-1].count_roots(**countKwargs)

	while contours:
		# yield the initial state here so that the animation in demo_find_roots shows the first frame
		totFoundRoots = sum(int(round(multiplicity.real)) for root, multiplicity in zip(roots, multiplicities))
		yield roots, multiplicities, contours, originalContour._numberOfRoots - totFoundRoots

		contour = contours.pop()

		if verbose: print(contour._numberOfRoots, 'roots in', contour)

		# if a known root is too near to this contour then reverse the subdivision that created it
		if np.any([contour.distance(root) < newtonStepTol for root in roots]):
			# remove the contour together with its children and siblings
			remove_siblings_children(contour)

			# put the parent contour back into the list of contours to be subdivided again
			contours.append(contour._parentContour)

			# do not use this contour again
			failedContours.append(contour)
			continue

		# if the contour is smaller than the newtonStepTol then just assume that
		# the root is at the center of the contour, print a warning and move on
		if contour.area < newtonStepTol:
			root = iterateToRoot(contour.centralPoint, f, df, newtonStepTol, rootErrTol, newtonMaxIter, attemptIterBest, verbose)
			if root is None or abs(f(root)) > abs(f(contour.centralPoint)) or not contour.contains(root):
				root = contour.centralPoint

			warnings.warn("The area of the interior of this contour with is smaller than newtonStepTol!  Try increasing rootTol" \
				"The point z = %f + %fi has been recorded as a root of multiplicity %i." \
				"The error |f(z)| = "%(root.real, contour.centralPoint.imag, contour._numberOfRoots) + str(abs(f(root))) \
				)
			addRoot(root,  contour._numberOfRoots)
			continue

		# if all the roots within the contour have been located then continue to the next contour
		numberOfKnownRootsInContour = sum([int(round(multiplicity.real)) for root, multiplicity in zip(roots, multiplicities) if contour.contains(root)])
		if contour._numberOfRoots == numberOfKnownRootsInContour:
			continue

		# if there are too many roots within the contour then subdivide or
		# if there are any known roots within the contour then also subdivide
		# (so as not to waste time re-approximating these roots)
		if numberOfKnownRootsInContour > 0 or contour._numberOfRoots > M:
			subdivide(contour)
			continue

		### Approximate the roots in this contour
		if intMethod == 'romb':
			# Check to see if the number of roots has changed after new values of f have been sampled
			def callback(I, err, numberOfDiv):
				if numberOfDiv > contour._numberOfDivisionsForN:
					if verbose: print('--- Checking N using the newly sampled values of f ---')
					new_N = contour.count_roots(f, df, NIntAbsTol=NIntAbsTol, integerTol=integerTol,
						divMin=numberOfDiv, divMax=divMax, m=m, intMethod=intMethod, verbose=verbose)
					if verbose: print('------------------------------------------------------')

					if new_N != contour._numberOfRoots:
						if verbose:
							print('N has been recalculated using more samples of f')
						contour._numberOfRoots = new_N
						raise NumberOfRootsChanged("""The additional function evaluations of f taken while
							approximating the roots within the contour have been shown that the number of roots
							of f within the contour is %i rather than the supplied %i."""%(new_N, contour._numberOfRoots))
		else:
			callback = None

		try:
			approxRoots, approxMultiplicities = contour.approximate_roots(contour._numberOfRoots, f, df,
				absTol=absTol, relTol=relTol, errStop=errStop, divMin=divMin, divMax=divMax,
				m=m, rootTol=newtonStepTol, intMethod=intMethod, callback=callback, verbose=verbose)
		except NumberOfRootsChanged:
			if verbose: print('The number of roots within the contour have been reevaluated.')
			if contour._numberOfRoots > M:
				subdivide(contour)
			else:
				contours.append(contour)
			continue

		for approxRoot, approxMultiplicity in list(zip(approxRoots, approxMultiplicities)):
			if verbose: print('approxRoot', approxRoot, 'approxMultiplicity', approxMultiplicity)

			# check that the multiplicity is close to an integer
			multiplicity = round(approxMultiplicity.real)
			if abs(multiplicity - approxMultiplicity.real) > integerTol or abs(approxMultiplicity.imag) > integerTol or multiplicity < 1:
				continue

			# attempt to refine the root
			root = iterateToRoot(approxRoot, f, df, newtonStepTol, rootErrTol, newtonMaxIter, attemptIterBest, verbose)

			if root is None or abs(f(approxRoot)) < abs(f(root)):
				# stick with the original approximation
				root = approxRoot

			if abs(f(root)) < rootErrTol:
				if np.any(np.abs(np.round(approxMultiplicities) - approxMultiplicities) > integerTol):
					# the computed multiplicity might be unreliable so make a contour focused on that point instead
					if hasattr(contour, '_shrinkingRadius'):
						contour._shrinkingRadius *= 0.5
						contours.append(Circle(root, contour._shrinkingRadius))
					else:
						contours.append(Circle(root, 1e-3))
						contours[-1]._shrinkingRadius = 1e-3
					contours[-1]._numberOfRoots = contours[-1].count_roots(**countKwargs)
				else:
					addRoot(root, multiplicity)

			# if the root turns out to be very close to the contour then this may have
			# introduced an error.  Therefore, compute the multiplicity of this root
			# directly and disregard this contour (repeat its parent's subdivision).
			if contour.distance(root) < newtonStepTol:
				# remove the contour and any relations
				remove_siblings_children(contour)

				# put the parent contour back into the list of contours to subdivide again
				parent = contour._parentContour
				contours.append(parent)

				# do not use this contour again
				failedContours.append(contour)

		# if we haven't found all the roots then subdivide further
		numberOfKnownRootsInContour = sum([int(round(multiplicity.real)) for root, multiplicity in zip(roots, multiplicities) if contour.contains(root)])
		if contour._numberOfRoots != numberOfKnownRootsInContour and contour not in failedContours:
			subdivide(contour)

	# delete cache for original contour incase this contour is being reused
	for segment in originalContour.segments:
		segment._integralCache = {}
		segment._trapValuesCache = {}

	if verbose:
		print('Completed rootfinding with', f.calls, 'evaluations of f at', f.points, 'points')
		print(RootResult(roots, multiplicities, originalContour))

	totFoundRoots = sum(int(round(multiplicity.real)) for root, multiplicity in zip(roots, multiplicities))
	yield roots, multiplicities, contours, originalContour._numberOfRoots - totFoundRoots

@docstrings.dedent
@doc_tab_to_space
@functools.wraps(find_roots_gen, assigned=('__module__', '__name__'))
def find_roots(originalContour, f, df=None, **kwargs):
	"""
	Find all the roots of the complex analytic function f within the
	given contour.

	Parameters
	----------
	%(find_roots_gen.parameters)s

	Returns
	-------
	result : :class:`RootResult <cxroots.RootResult.RootResult>`
		A container for the roots and their multiplicities.
	"""
	rootFinder = find_roots_gen(originalContour, f, df, **kwargs)
	for roots, multiplicities, contours, numberOfRemainingRoots in rootFinder:
		pass
	return RootResult(roots, multiplicities, originalContour)




from __future__ import division
import numpy as np

from ..Contour import Contour
from ..Paths import ComplexLine

class Rectangle(Contour):
	"""
	A positively oriented rectangle in the complex plane.

	Parameters
	----------
	xRange : tuple
		Tuple of length two giving the range of the rectangle along the
		real axis.
	yRange : tuple
		Tuple of length two giving the range of the rectangle along the
		imaginary axis.

	Examples
	--------
	.. plot::
		:include-source:

		from cxroots import Rectangle
		rect = Rectangle(xRange=(-2, 2), yRange=(-1, 1))
		rect.show()
	"""
	def __init__(self, xRange, yRange):
		self.xRange = xRange
		self.yRange = yRange
		self.axisName = ('x', 'y')

		self.z1 = z1 = self.xRange[0] + 1j*self.yRange[0]
		self.z2 = z2 = self.xRange[1] + 1j*self.yRange[0]
		self.z3 = z3 = self.xRange[1] + 1j*self.yRange[1]
		self.z4 = z4 = self.xRange[0] + 1j*self.yRange[1]

		segments = [ComplexLine(z1,z2),
					ComplexLine(z2,z3),
					ComplexLine(z3,z4),
					ComplexLine(z4,z1)]
		super(Rectangle, self).__init__(segments)

	def __str__(self):
		return 'Rectangle: vertices = {z1.real:.3f}{z1.imag:+.3f}i, {z2.real:.3f}{z2.imag:+.3f}i, {z3.real:.3f}{z3.imag:+.3f}i, {z4.real:.3f}{z4.imag:+.3f}i'.format(z1=self.z1, z2=self.z2, z3=self.z3, z4=self.z4)

	@property
	def centralPoint(self):
		# get the central point within the contour
		x = (self.xRange[0] + self.xRange[1])/2
		y = (self.yRange[0] + self.yRange[1])/2
		return x + 1j*y

	@property
	def area(self):
		return (self.xRange[1]-self.xRange[0])*(self.yRange[1]-self.yRange[0])

	def contains(self, z):
		""" Returns True if the point z lies within the contour, False if otherwise """
		return self.xRange[0] < z.real < self.xRange[1] and self.yRange[0] < z.imag < self.yRange[1]

	def subdivide(self, axis, divisionFactor=0.5):
		"""
		Subdivide the contour

		Parameters
		----------
		axis : str, can be either 'x' or 'y'
			The axis along which the line subdividing the contour is a
			constant.
		divisionFactor : float in range (0,1), optional
			Determines the point along 'axis' at which the line dividing
			the contour is placed.

		Returns
		-------
		box1 : Rectangle
			If axis is 'x' then box1 has the same yRange and minimum value of xRange as the
			original Rectangle but the maximum xRange is determined by the divisionFactor.
			If axis is 'y' then box1 has the same xRange and minimum value of yRange as the
			original Rectangle but the maximum yRange is determined by the divisionFactor.
		box2 : Rectangle
			If axis is 'x' then box2 has the same yRange and maximum value of xRange as the
			original Rectangle but the minimum xRange is equal to the maximum xRange of box1.
			If axis is 'x' then box2 has the same xRange and maximum value of yRange as the
			original Rectangle but the minimum yRange is equal to the maximum yRange of box1.
		"""
		if axis == 'x' or self.axisName[axis] == 'x':
			midpoint = self.xRange[0] + divisionFactor*(self.xRange[1]-self.xRange[0])
			box1 = Rectangle([self.xRange[0], midpoint], self.yRange)
			box2 = Rectangle([midpoint, self.xRange[1]], self.yRange)

			box1.segments[3] = self.segments[3]
			box2.segments[1] = self.segments[1]
			box1.segments[1]._reversePath = box2.segments[3]
			box2.segments[3]._reversePath = box1.segments[1]

		elif axis == 'y' or self.axisName[axis] == 'y':
			midpoint = self.yRange[0] + divisionFactor*(self.yRange[1]-self.yRange[0])
			box1 = Rectangle(self.xRange, [self.yRange[0], midpoint])
			box2 = Rectangle(self.xRange, [midpoint, self.yRange[1]])

			box1.segments[0] = self.segments[0]
			box2.segments[2] = self.segments[2]
			box1.segments[2]._reversePath = box2.segments[0]
			box2.segments[0]._reversePath = box1.segments[2]

		for box in [box1, box2]:
			box._createdBySubdivisionAxis = axis
			box._parentBox = self
		self._childBoxes = [box1, box2]

		return box1, box2

	def randomPoint(self):
		"""Returns a random point inside the contour of the Rectangle."""
		x = np.random.uniform(*self.xRange)
		y = np.random.uniform(*self.yRange)
		return x + 1j*y



#!/usr/bin/env python
from setuptools import setup

packages = ['cxroots', 'cxroots.tests', 'cxroots.contours']

# get the version, this will assign __version__
exec(open('cxroots/version.py').read())

# read the README_pip.rst
try:
    with open('README.rst') as file:
        long_description = file.read()
except IOError:
    long_description = None

setup(
    name = 'cxroots',
    version = __version__,
    description = 'Find all the roots (zeros) of a complex analytic function within a given contour in the complex plane.',
    long_description = long_description,
    author = 'Robert Parini',
    author_email = 'robert.parini@gmail.com',
    url = 'https://rparini.github.io/cxroots/',
    license = 'BSD',
    data_files = [("", ["LICENSE"])],
    packages = packages,
    zip_safe = False,   # prevent cxroots from installing as a .egg zip file
    platforms = ['all'],
    dependency_links=['git+https://github.com/rparini/numdifftools.git@fix_vstack_futurewarning#egg=numdifftools'],
    setup_requires = ['pytest-runner'],
    install_requires = ['numpy', 'scipy', 'docrep', 'mpmath', 'numdifftools'],
    tests_require=['pytest'],
    keywords='roots zeros complex analytic functions',
    classifiers=[
	    'Development Status :: 4 - Beta',
	    'Intended Audience :: Science/Research',
	    'Topic :: Scientific/Engineering :: Mathematics',
	    'License :: OSI Approved :: BSD License',
	    'Programming Language :: Python :: 2.7',
	    'Programming Language :: Python :: 3',
	],
)



from __future__ import division
import numpy as np
from scipy import pi

from ..Contour import Contour
from ..Paths import ComplexArc
from .Annulus import Annulus

class Circle(Contour):
	"""
	A positively oriented circle in the complex plane.

	Parameters
	----------
	center : complex
		The center of the circle.
	radius : float
		The radius of the circle.

	Examples
	--------
	.. plot::
		:include-source:

		from cxroots import Circle
		circle = Circle(center=1, radius=0.5)
		circle.show()
	"""
	def __init__(self, center, radius):
		self.center = center
		self.radius = radius
		self.axisName = ('r')

		segments = [ComplexArc(center, radius, 0, 2*pi)]
		super(Circle, self).__init__(segments)

	def __str__(self):
		return 'Circle: center={center.real:.3f}{center.imag:+.3f}i, radius={radius:.3f}'.format(center=self.center, radius=self.radius)

	def contains(self, z):
		""" Returns True if the point z lies within the contour, False if otherwise """
		return abs(z - self.center) < self.radius

	@property
	def centralPoint(self):
		return self.center

	@property
	def area(self):
		return pi*self.radius**2

	def subdivide(self, axis='r', divisionFactor=0.5):
		"""
		Subdivide the contour

		Parameters
		----------
		axis : str, can only be 'r' (argument kept for consistency with 'subdivisions' method in parent Contour class)
			The axis along which the line subdividing the contour is a constant.
		divisionFactor : float in range (0,1), optional
			Determines the point along 'axis' at which the line dividing the box is placed

		Returns
		-------
		box1 : Annulus
			With inner radius determined by the divisionFactor and outer radius equal to that of the original circle
		box2 : Circle
			With radius equal to the inner radius of box1
		"""
		if axis == 'r' or self.axisName[axis] == 'r':
			box1 = Annulus(self.center, [self.radius*divisionFactor, self.radius])
			box2 = Circle(self.center, self.radius*divisionFactor)
			box1.segments[0] = self.segments[0]
			box1.segments[1]._reversePath = box2.segments[0]
			box2.segments[0]._reversePath = box1.segments[1]

		for box in [box1, box2]:
			box._createdBySubdivisionAxis = axis
			box._parentBox = self
			self._childBoxes = [box1, box2]

		return box1, box2

	def randomPoint(self):
		""" Returns a random point inside the Circle """
		r   = np.random.uniform(0,self.radius)
		phi = np.random.uniform(0,2*pi)
		return r*exp(1j*phi) + self.center



import pytest

from cxroots.IterativeMethods import muller
from scipy import pi, cos

def test_muller():
	f  = lambda x: cos(x) - x**2 + 1j*x**3

	iterations = []
	callback = lambda x, dx, y, iteration: iterations.append(x)
	x, err = muller(0.5, pi/4, 0.6, f)

	root = 0.7296100078977741539157356847 + 0.1570923181734581909733787621j
	assert x == pytest.approx(root)




import pytest

from cxroots.IterativeMethods import newton
from scipy import pi, cos, sin

def test_newton():
	# result from keisan online calculator: http://keisan.casio.com/exec/system/1244946907
	f  = lambda x: cos(x)-x
	df = lambda x: -sin(x)-1

	iterations = []
	callback = lambda x, dx, y, iteration: iterations.append(x)
	x, err = newton(pi/4, f, df, callback=callback)
	iterations.append(x)
	
	correct_iterations = [0.73953613351523830094,
						  0.7390851781060101829533,
			  			  0.7390851332151610866198,
			  			  0.7390851332151606416553]

	assert iterations == pytest.approx(correct_iterations)




from __future__ import division

import numpy as np
import scipy.integrate
from scipy import exp, pi

class ComplexPath(object):
	"""A base class for paths in the complex plane."""
	def __init__(self):
		self._integralCache = {}
		self._trapValuesCache = {}


	def __call__(self, t):
		r"""
		The parameterization of the path in the varaible :math:`t\in[0,1]`.

		Parameters
		----------
		t : float
			A real number :math:`0\leq t \leq 1`.

		Returns
		-------
		complex
			A point on the path in the complex plane.
		"""
		raise NotImplementedError('__call__ must be implemented in a subclass')


	def trap_values(self, f, k, useCache=True):
		"""
		Compute or retrieve (if cached) the values of the functions f
		at :math:`2^k+1` points along the contour which are evenly
		spaced with respect to the parameterisation of the contour.

		Parameters
		----------
		f : function
			A function of a single complex variable.
		k : int
			Defines the number of points along the curve that f is to be
			evaluated at as :math:`2^k+1`.
		useCache : bool, optional
			If True then use, if available, the results of any previous
			calls to this function for the same f and save any new
			results so that they can be reused later.

		Returns
		-------
		:class:`numpy.ndarray`
			The values of f at :math:`2^k+1` points along the contour
			which are evenly spaced with respect to the parameterisation
			of the contour.
		"""
		if f in self._trapValuesCache.keys() and useCache:
			vals = self._trapValuesCache[f]
			vals_k = int(np.log2(len(vals)-1))

			if vals_k == k:
				return vals
			elif vals_k > k:
				return vals[::2**(vals_k-k)]
			else:
				t = np.linspace(0, 1, 2**k+1)
				vals = np.empty(2**k+1, dtype=np.complex128)
				vals.fill(np.nan)
				vals[::2**(k-vals_k)] = self._trapValuesCache[f]
				vals[np.isnan(vals)] = f(self(t[np.isnan(vals)]))

				# cache values
				self._trapValuesCache[f] = vals
				return vals

		else:
			t = np.linspace(0, 1, 2**k+1)
			vals = f(self(t))
			if useCache:
				self._trapValuesCache[f] = vals
			return vals


	def plot(self, N=100, linecolor='C0', linestyle='-'):
		"""
		Uses matplotlib to plot, but not show, the path as a 2D plot in
		the Complex plane.

		Parameters
		----------
		N : int, optional
			The number of points to use when plotting the path.
		linecolor : optional
			The colour of the plotted path, passed to the
			:func:`matplotlib.pyplot.plot` function as the keyword
			argument of 'color'.  See the matplotlib tutorial on
			`specifying colours <https://matplotlib.org/users/colors.html#>`_.
		linestyle : str, optional
			The line style of the plotted path, passed to the
			:func:`matplotlib.pyplot.plot` function as the keyword
			argument of 'linestyle'.  The default corresponds to a solid
			line.  See :meth:`matplotlib.lines.Line2D.set_linestyle` for
			other acceptable arguments.
		"""
		import matplotlib.pyplot as plt
		t = np.linspace(0,1,N)
		path = self(t)
		plt.plot(path.real, path.imag, color=linecolor, linestyle=linestyle)
		plt.xlabel('Re[$z$]', size=16)
		plt.ylabel('Im[$z$]', size=16)
		plt.gca().set_aspect(1)

		# add arrow to indicate direction of path
		arrow_direction = (self(0.51) - self(0.5))/abs(self(0.51) - self(0.5))
		arrow_extent = 1e-6*arrow_direction
		ymin, ymax = plt.gca().get_ylim()
		xmin, xmax = plt.gca().get_xlim()
		head_length = max(abs(ymax - ymin), abs(xmax - xmin))/40.
		plt.arrow(self(0.5).real, self(0.5).imag,
				  arrow_extent.real, arrow_extent.imag,
				  head_width=head_length*2/3., head_length=head_length,
				  fc=linecolor, ec=linecolor)

	def show(self, saveFile=None, **plotKwargs):
		"""
		Shows the path as a 2D plot in the complex plane.  Requires
		Matplotlib.

		Parameters
		----------
		saveFile : str (optional)
			If given then the plot will be saved to disk with name
			'saveFile'.  If saveFile=None the plot is shown on-screen.
		**plotKwargs
			Other key word arguments are passed to :meth:`~cxroots.Paths.ComplexPath.plot`.
		"""
		import matplotlib.pyplot as plt
		self.plot(**plotKwargs)

		if saveFile is not None:
			plt.savefig(saveFile, bbox_inches='tight')
			plt.close()
		else:
			plt.show()

	def integrate(self, f, absTol=0, relTol=1e-12, divMax=15, intMethod='quad', verbose=False):
		"""
		Integrate the function f along the path.  The value of the
		integral is cached and will be reused if the method is called
		with same arguments (ignoring verbose).

		Parameters
		----------
		f : function
			A function of a single complex variable.
		absTol : float, optional
			The absolute tolerance for the integration.
		relTol : float, optional
			The realative tolerance for the integration.
		divMax : int, optional
			If the Romberg integration method is used then divMax is the
			maximum number of divisions before the Romberg integration
			routine of a path exits.
		intMethod : {'quad', 'romb'}, optional
			If 'quad' then :func:`scipy.integrate.quad` is used to
			compute the integral.  If 'romb' then Romberg integraion,
			using :func:`scipy.integrate.romberg`, is used instead.
		verbose : bool, optional
			Passed ass the `show` argument of :func:`scipy.integrate.romberg`.

		Returns
		-------
		complex
			The integral of the function f along the path.

		Notes
		-----
		This function is only used when checking the
		multiplicity of roots.  The bulk of the integration for
		rootfinding is done with :func:`cxroots.CountRoots.prod`.
		"""

		args = (f, absTol, relTol, divMax, intMethod)
		if args in self._integralCache.keys():
			integral = self._integralCache[args]

		elif hasattr(self, '_reversePath') and args in self._reversePath._integralCache:
			# if we have already computed the reverse of this path
			integral = -self._reversePath._integralCache[args]

		else:
			integrand = lambda t: f(self(t))*self.dzdt(t)

			if intMethod == 'romb':
				integral = scipy.integrate.romberg(integrand, 0, 1, tol=absTol, rtol=relTol, divmax=divMax, show=verbose)
			elif intMethod == 'quad':
				integrand_real = lambda t: np.real(integrand(t))
				integrand_imag = lambda t: np.imag(integrand(t))

				integral_real, abserr_real = scipy.integrate.quad(integrand_real, 0, 1, epsabs=absTol, epsrel=relTol)
				integral_imag, abserr_imag = scipy.integrate.quad(integrand_imag, 0, 1, epsabs=absTol, epsrel=relTol)
				integral = integral_real + 1j*integral_imag
			else:
				raise ValueError("intMethod must be either 'romb' or 'quad'")

			if np.isnan(integral):
				raise RuntimeError('The integral along the segment %s is NaN.\
					\nThis is most likely due to a root being on or very close to the path of integration.'%self)

			self._integralCache[args] = integral

		return integral


class ComplexLine(ComplexPath):
	r"""
	A straight line :math:`z` in the complex plane from a to b
	parameterised by

	..math::

		z(t) = a + (b-a)t, \quad 0\leq t \leq 1


	Parameters
	----------
	a : float
	b : float
	"""
	def __init__(self, a, b):
		self.a, self.b = a, b
		self.dzdt = lambda t: self.b-self.a
		super(ComplexLine, self).__init__()

	def __str__(self):
		return 'ComplexLine from %.3f+%.3fi to %.3f+%.3fi' % (self.a.real, self.a.imag, self.b.real, self.b.imag)

	def __call__(self, t):
		r"""
		The function :math:`z(t) = a + (b-a)t`.

		Parameters
		----------
		t : float
			A real number :math:`0\leq t \leq 1`.

		Returns
		-------
		complex
			A point on the line in the complex plane.
		"""
		return self.a + t*(self.b-self.a)

	def distance(self, z):
		"""
		Distance from the point z to the closest point on the line.

		Parameters
		----------
		z : complex

		Returns
		-------
		float
			The distance from z to the point on the line which is
			closest to z.
		"""
		# convert complex numbers to vectors
		A = np.array([self.a.real, self.a.imag])
		B = np.array([self.b.real, self.b.imag])
		Z = np.array([z.real, z.imag])

		# the projection of the point z onto the line a -> b is where
		# the parameter t is
		t = (Z-A).dot(B-A)/abs((B-A).dot(B-A))

		# but the line segment only has 0 <= t <= 1
		t = t.clip(0,1)

		# so the point on the line segment closest to z is
		c = self(t)
		return abs(c-z)

class ComplexArc(ComplexPath):
	r"""
	A circular arc :math:`z` with center z0, radius R, initial angle t0
	and change of angle dt.  The arc is parameterised by

	..math::

		z(t) = R e^{i(t0 + t dt)} + z0, \quad 0\leq t \leq 1

	Parameters
	----------
	z0 : complex
	R : float
	t0 : float
	dt : float
	"""
	def __init__(self, z0, R, t0, dt):
		self.z0, self.R, self.t0, self.dt = z0, R, t0, dt
		self.dzdt = lambda t: 1j*self.dt*self.R*exp(1j*(self.t0 + t*self.dt))
		super(ComplexArc, self).__init__()

	def __str__(self):
		return 'ComplexArc: z0=%.3f, R=%.3f, t0=%.3f, dt=%.3f' % (self.z0, self.R, self.t0, self.dt)

	def __call__(self, t):
		r"""
		The function :math:`z(t) = R e^{i(t_0 + t dt)} + z_0`.

		Parameters
		----------
		t : float
			A real number :math:`0\leq t \leq 1`.

		Returns
		-------
		complex
			A point on the arc in the complex plane.
		"""
		return self.R*exp(1j*(self.t0 + t*self.dt)) + self.z0

	def distance(self, z):
		"""
		Distance from the point z to the closest point on the arc.

		Parameters
		----------
		z : complex

		Returns
		-------
		float
			The distance from z to the point on the arc which is closest
			to z.
		"""
		theta = np.angle(z-self.z0) 				# np.angle maps to (-pi,pi]
		theta = (theta-self.t0)%(2*pi) + self.t0 	# put theta in [t0,t0+2pi)

		if ((self.dt > 0 and self.t0 < theta < self.t0+self.dt)
			or (self.dt < 0 and self.t0+self.dt < theta - 2*pi < self.t0)):
			# the closest point to z lies on the arc
			return abs(self.R*exp(1j*theta) + self.z0 - z)
		else:
			# the closest point to z is one of the endpoints
			return min(abs(self(0)-z), abs(self(1)-z))




import pytest
import numpy as np

from cxroots import Circle, Rectangle
from cxroots.tests.ApproxEqual import roots_approx_equal

funcs = [
	pytest.param([3, 3.01, 3.02, 8, 8.02, 8+0.01j], 			[1,1,1,1,1,1], id='cluster_10^-2'),
	pytest.param([3, 3.001, 3.002, 8, 8.002, 8+0.001j], 		[1,1,1,1,1,1], id='cluster_10^-3'),
	pytest.param([3, 3.0001, 3.0002, 8, 8.0002, 8+0.0001j], 	[1,1,1,1,1,1], id='cluster_10^-4', marks=pytest.mark.slow),
	pytest.param([3, 3.00001, 3.00002, 8, 8.00002, 8+0.00001j], [1,1,1,1,1,1], id='cluster_10^-5', marks=[pytest.mark.slow, pytest.mark.xfail(reason='Cluster of roots too tight')])
]

contours = [
	pytest.param(Rectangle([2,9], [-1,1]), id='rect'),
	pytest.param(Circle(0, 8.5), id='circle')
]

@pytest.mark.parametrize('C', contours)
@pytest.mark.parametrize('roots,multiplicities', funcs)
def test_rootfinding_df(C, roots, multiplicities):
	f  = lambda z: np.prod([z-r for r in roots], axis=0)
	df = lambda z: np.sum([np.prod([z-r for r in np.delete(roots,i)], axis=0) for i in range(len(roots))], axis=0)
	roots_approx_equal(C.roots(f, df, verbose=True), (roots, multiplicities))

@pytest.mark.slow
@pytest.mark.parametrize('C', contours)
@pytest.mark.parametrize('roots,multiplicities', funcs)
def test_rootfinding_f(C, roots, multiplicities):
	f  = lambda z: np.prod([z-r for r in roots], axis=0)
	roots_approx_equal(C.roots(f, verbose=True), (roots, multiplicities))




import pytest
import numpy as np
from scipy import cos, sin

from cxroots import Circle, Rectangle
from cxroots import CxDerivative

@pytest.mark.parametrize('C', [
	pytest.param(Circle(0, 2), id='circle'),
	pytest.param(Rectangle([-1.5,1.5],[-2,2]), id='rect'),
	pytest.param(None, id='default')
])
def test_CxDerivative(C):
	f  = lambda z: z**10 - 2*z**5 + sin(z)*cos(z/2)
	df = lambda z: 10*(z**9 - z**4) + cos(z)*cos(z/2) - 0.5*sin(z)*sin(z/2)

	z = np.array([-1.234, 0.3+1j, 0.1j, -0.9-0.5j])

	assert CxDerivative(f, z, n=1, contour=C) == pytest.approx(df(z))

